; The program (an infinite loop) keeps track of time
; using software delays.


; SYMBOLIC CONSTANTS
LCD_CLEAR_COMMAND equ 01
LCD_CMD           equ $1400
LCD_STATUS        equ LCD_CMD
LCD_DATA        equ LCD_CMD+1


; THE VARIABLES
; Modifiable data -- For stand-alone programs we start this segment
; (by convention) at 0x7000
              org $7000
tenths        rmb 1  ;a binary number (signed or unsigned) > 0 and <= 9
seconds       rmb 1  ;Software maintains this variable in packed BCD format
minutes       rmb 1  ;Software maintains this variable in packed BCD format

asciiTime:
asciiMinutes  rmb 2  ;Slot to hold minutes (ASCII)
              fcc ':'
asciiSeconds  rmb 2  ;Slot to hold seconds (ASCII)
              fcb 0  ;string null terminator

; THE PROGRAM
; The program is NOT self-modifying; it is placed in a separate memory
; segment that could be ROM.
; By convention, this segment begins at 0x6000.
; By convention, the first executable statement is labelled "main".

              org $6000
main:         jsr init          ;"tenths", "seconds" and "minutes" set to zero

; The main infinite loop implements the following algorithm:
;  while(1) {
;      wait 1/10 second and increment "tenths"
;      if "tenths" == 10  { //i.e. 1 second has elapsed
;         set "tenths" to zero
;         incrementSecondsMinutes()
;         updateLCD()
;      }
;  }

forever:
      ldx #100
      jsr delayXms  ;delay 100,000 clock cycles (1/10 sec)
      ldaa tenths
      inca
      staa tenths
      cmpa #10
      blo forever
;one second has elapased if we get here
      clr tenths
      jsr incrementSecondsMinutes
      jsr updateLCD
      jmp forever
      stop    ;NEVER (we hope) executed



;"delay1ms" subroutine implements a 1000 cycle software loop
DEL_COUNT      equ  193
delay1ms       psha             ; 3 cycles
               tpa              ; 2 cycles
               psha             ; 3 cycles
               ldaa #DEL_COUNT  ; 2 cycles
delay1msLoop   deca             ; 2 cycles
               bne delay1msLoop ; 3 cycles
               pula             ; 4 cycles
               tap              ; 2 cycles
               pula             ; 4 cycles
               nop              ; 2 cycles (PADDING)
               nop              ; 2 cycles (PADDING)
               rts              ; 5 cycles
;"init" sets things up
init:
              clr tenths
              clr seconds
              clr minutes
              jsr clearLCD
              rts

clearLCD:
             ldaa #LCD_CLEAR_COMMAND
             jsr doLCDCommand
             rts


doLCDCommand:
             jsr LCDReady
             staa LCD_CMD
             rts

LCDReady:
             tst LCD_STATUS
             bmi LCDReady
             rts

updateLCD:
            jsr updateAscii

            rts

updateAscii:
            ldaa seconds
            jsr packedBCDToAscii
            std asciiSeconds
            ldaa minutes
            jsr packedBCDToAscii
            std asciiMinutes
            jsr clearLCD
            ldx #asciiTime
            ldaa 0,x
            jsr displayLCD
            inx
            ldaa 0,x
            jsr displayLCD
            inx
            ldaa 0,x
            jsr displayLCD
            inx
            ldaa 0,x
            jsr displayLCD
            inx
            ldaa 0,x
            jsr displayLCD
            rts


packedBCDToAscii:
            tab
            lsra
            lsra
            lsra
            lsra
            andb #$0f
            addd #$3030
            rts

displayLCD:
            jsr LCDReady
            staa LCD_DATA
            rts


;"incrementSecondsMinutes" implements the algorithm:
;  seconds++
;  if (seconds == 60) {
;      seconds = 0;
;      minutes++;
;  }
; The arithmetic is performed in Packed BCD format.
incrementSecondsMinutes:
       ldaa seconds
       adda #1
       daa
       staa seconds
       cmpa #$60
       bne iSecMinDone
       clr seconds
       ldaa minutes
       adda #1
       daa
       staa minutes
iSecMinDone:
       rts

;; @name delayXms
;; Software delay loop for about X ms.  (Actually,
;; 1000*X E-clock cycles; with a 1 MHz E-clock, this means X milliseconds.)
;; If interrupts are
;; enabled, the actual time between subroutine invocation and return
;; may be greater.  With no interrupts, the delay is a bit more than X
;; ms.  (For large X, the delay will be no more than 1% greater than
;; specificed.)
;;
;; <b>Note:</b> If X is zero, it does <em>not</em> mean a short delay!
;; It means delay by 65536 ms, ie. more than a minute.
;;
;; <b>Note:</b> For accurate time delays, use the
;; timer module (with or without interrupts.)
;;
;; @param IX number of clock cycles to delay
;; @return nothing
;; @side none
;;

; n_cycles = overhead + x*1006 = 27 + 1006*x
delayXms:
;prologue---save registers
	       psha             ; 3 cycles
               tpa              ; 2 cycles
               psha             ; 3 cycles
	       pshx             ; ?4 cycles
DXM_Loop   jsr delay1ms     ; 1000 cycles
               dex          ; ?3 cycles
               bne DXM_Loop ; 3 cycles
;epilogue---restore registers
               pulx             ; ?5 cycles
               pula             ; 4 cycles
               tap              ; 2 cycles
               pula             ; 4 cycles
               rts              ; 5 cycles

           end
