package com.kclowes.sim11;
import java.util.*;

/**
 * This abstract class provides a skeleton implemention
 * of a byte-sized {@link Cell Cell}.
 *
 * @version 1.3 (August 2, 2002)
 * @author Ken Clowes
 */
public class AbstractByteCell implements Cell
{
    /** Contents of a byte. Only the 8 low order bits are significant
     * and the the upper bits SHOULD all be zero.  (Perhaps add
     * some assert statments to verify this.)
     */
    public int contents;
    public int nWrites;
    public int nReads;
    public int address;
    public ArrayList listeners = new ArrayList();
    public  final static int DEFAULT_CONTENTS = 0xff;
    
    
    public CPU6811 cpu =  CPU6811.getInstance();
    /** Creates a byte with known address and initial contents.
     * @param address The address where the byte is mapped.
     * @param contents The initial contents of the byte.
     */
    
    public AbstractByteCell(int  address, int contents)
    {
	this.address = address;
	this.contents = contents;
	nWrites = 0;
	nReads = 0;
    }
    
    /** Creates a byte with known address.
     * @param address The address where the byte is mapped.
     */
    public AbstractByteCell(int address)
    {
	this(address, DEFAULT_CONTENTS);
    }
    
    /** Returns the contents of the byte by simulating a bus read
     * cycle.  The number of times the cell is read is tracked.
     */
    public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  contents;
    }
    
    /** Returns the contents of the byte.  There is no tracking of the
     * number of times the cell contents are obtained. 
     */
    public int get()
    {
	return  contents;
    }
    
    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {
	nWrites++;
	cpu.incrementWriteCycles();
	set(contents);
    }
    
    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written is tracked.
     *
     * @param contents What is written to the byte.
     */
    public void write(byte contents)
    {
	write(((int) contents)&0xff);
    }
    
    /** Set the contents of the byte.  If the contents have changed,
     * all registered CellChangeListeners are invoked through their
     * <code>perform()</code> method.
     *
     * @param newContents The new contents of the byte.
     */
    
    public void set(int newContents)
    {
	if (newContents != contents) {
	    contents = newContents;
	    if (listeners.size() != 0) {
		for(Iterator iter = listeners.iterator();
		    iter.hasNext();) {
		    ((CellChangeListener) iter.next()).perform(this);
		}
	    }
	    return;
	}
	contents = newContents;
	
    }
    
    /** Reset the byte contents and statistics */
    public void reset()
    {
	nReads = 0;
	nWrites = 0;
	contents = DEFAULT_CONTENTS;
    }
    
    /** Get the address of the Cell. */
    public int getAddress()
    {
	return address;
    }

    /** Adds a CellChangeListerer to the cell. */
    public void addCellChangeListener(CellChangeListener l)
    {
	listeners.add(l);
    }
    
    /** Remove a CellChangeListener from the Cell. */
    public boolean removeCellChangeListener(CellChangeListener l)
    {
	boolean removed = false;
	for(Iterator iter = listeners.iterator();
	    iter.hasNext();) {
	    if (iter.next() == l) {
		iter.remove();
		removed = true;
		break;
	    }
	}
	return removed;
    }
}
