package com.kclowes.sim11;
/** The root interface of <em>cells</em>, the building blocks of a
 * Memory System.  Note the difference between reading (writing) and
 * getting (setting) a cell.  <em>Getting</em> the Cell contents
 * returns the current value stored in the Cell but does not simulate
 * a read bus cycle or update statistics about Cell usage whereas
 * <em>reading</em> a Cell does perform these operations.  The
 * difference between <em>setting</em> and <em>writing</em> is even
 * greater.  For example, a Read Only Memory Cell will not change
 * the Cell contents when <em>write</em> is invoked.  However, the
 * <em>set</em> method allows the ROM Cell to be initialized to the
 * desired value.
 *
 * <p>
 * Broadly speaking, the running simulator uses read and write methods
 * to simulate program execution; the set and get methods are used by
 * the user interface to initialize or display Cells.  Devices that
 * that are made up of multiple special Cells also typcially use get
 * and set methods for inter-register communication.  (For example,
 * suppose a device ignores writes to its data register if its status
 * register indicates that it is busy.  The Cell implementing the data
 * register could query the status register Cell with a <em>get</em>
 * method to determine how it should simulate a <em>write</em> call.
 * </p>
 *
 * @see MemorySpace
 * @version 1.2 (June 20, 2002)
 * @author Ken Clowes
 */
public interface Cell {
    /** Write an integer to the cell. If the cell width is less than
     * 32, the low order bits are written.
     *
     * @param contents What is written into the cell.
     */
    void write(int contents);

    /** Write a byte to the cell.
     *
     * @param contents What is written into the cell.
     */
    void write(byte contents);

    /** Read the cell and return the contents. */
    int  read();

    /** Get the contents of the Cell. */
    int get();
    
    /** Set the contents of the Cell. */
    void set(int contents);

    /** Reset the Cell contents and statistics */
    void reset();

    /** Get the address of the Cell. */
    int getAddress();

    /** Add a CellChangeListener to the Cell. */
    void addCellChangeListener(CellChangeListener l);

    /** Remove a CellChangeListener from the Cell.
     *
     * @return true if removed
     */
    boolean removeCellChangeListener(CellChangeListener l);

}
