package com.kclowes.sim11;
import java.util.*;
/** This class maintains a delta time list of objects that implement
 * the DeltaTimed interface.
 * <p>
 * Items added to the list specify a time (number of ticks from now)
 * when they should be removed from the list.  For example, suppose
 * that initially the following items are added to the list:
 *
 * <bl>
 * <li> Foo (to fire after 5 ticks)</li>
 * <li> Bar (to fire after 8 ticks)</li>
 * </bl>
 * </p>
 * <p>
 * Suppose that after 3 ticks have elapsed, two other items are added:
 * <bl>
 * <li> Goo (to fire after 4 ticks)</li>
 * <li> Moo (to fire after 1 tick)</li>
 * </bl>
 * </p>
 *
 * <p>
 * The Foo, Bar, Goo and Moo events should now be fired in the order:
 * <bl>
 *  <li> Moo should fire after 1 tick.</li>
 *  <li> Foo should fire after 2 ticks.</li>
 *  <li> Goo should fire after 4 ticks.</li>
 *  <li> Bar should fire after 5 ticks.</li>
 * </bl>
 * 
 * @version 1.1 (September 23, 2002)
 * @author Ken Clowes
 */


public class DeltaList {

    private LinkedList list = new LinkedList();
    
    /** Create a DeltaList */
    public DeltaList()
    {
    }
    
    /** Number of items in the list. */
    public int size()
    {
	return list.size();
    }

    public void add(DeltaTimed item)
    {
	//special case: adding to empty list (should be incorp'ed into algo
	if(list.size() == 0) {
	    list.add(item);
	    return;
	}

	// Initial delta time for item being added
	int itemAddTime = item.getDelta();
	// Step thru list
	ListIterator listIterator = list.listIterator();
	for(int i = 0; listIterator.hasNext(); i++) {
	    DeltaTimed nextItem = (DeltaTimed) listIterator.next();
	    int dTime = nextItem.getDelta();
	    itemAddTime -= dTime;
	    // itemAddTime now decremented by delta time of next
	    // item in delta list...
	    // Keep on going if the decremented delta > 0
            // Otherwise...following if statement
	    if (itemAddTime <= 0) {
		// First, ensure we have a positive new delta time
		if (itemAddTime != 0) {
		    itemAddTime += dTime;
		}
		//...and set the delta of the item being added accordingly
		item.setDelta(itemAddTime);
		if ((i == 0) && (itemAddTime == 0)) {
		    list.add(1, item);
		} else {
		    // Second, decrement delta time of "current next item"
		    // to make room for new inserted item UNLESS we're
		    // at head of list.
		    nextItem.setDelta(dTime - itemAddTime);
		    // Finally, insert the new item into the delta list
		    list.add(i, item);
		    // We're done: RETURN NOW
		}
		return;
	    }
	}
	item.setDelta(itemAddTime);
	list.add(item);
	return;
    }

//     /** Get first item on delta list.
//      */
//     public DeltaTimed get()
//     {
// 	if (list.size() == 0) {
// 	    return null;
// 	}
// 	return (DeltaTimed) list.get(0);
//     }


    /** Determine if the next item is due now. (i.e. Is tick zero?)
     */
    public final boolean isNextNow()
    {
	if (list.size() == 0) {
	    return false;
	}

	DeltaTimed item = (DeltaTimed) list.get(0);
	int ticks = item.getDelta();
	return (ticks == 0);
    }

    /** Decrement tick count on next item and return it if ticks == 0.
     * If the head has a delta of 0, remove and return it without
     * decrementing ticks.
     * @return expired item or null if none
     */

    public final DeltaTimed tickAndRemove()
    {
	if (list.size() == 0) {
	    return null;
	}

	DeltaTimed item = (DeltaTimed) list.get(0);
	int ticks = item.getDelta();
	if (ticks == 0) {
	    list.remove(0);
	    return item;
	}
	ticks--;
	//KLUDGE
	if (ticks < 0) {
	    CPU6811 cpu = CPU6811.getInstance();
	    System.out.println("Ticks < 0 !!!!!; cycles: " + cpu.getNCycles());
	    ticks = 0;//SHOULD NOT HAPPEN
	}
	item.setDelta(ticks);
	if (ticks == 0) {
	    list.remove(0);
	    return item;
	}
	return null;
    }

    /** Returns the string representation of a DeltaList.
     *
     *<p>
     *The format of the serialized (String) representation is:
     *
     * <pre>
     * [ DELTA_VALUE (, DELTA_VALUE)* ]
     *  </pre>
     * where DELTA_VALUE is the 
     *</p>
     *
     *
     */
    public String toString() {
	String s;
	s = "[";
	for(int i = 0; i < size(); i++) {
	    if (i >=1) {
		s += ", ";
	    }
	    String itemDesc;
	    itemDesc = ((TimedEvent)((DeltaTimed)list.get(i))).toString();
	    s += itemDesc;
	    
	}
	return s+"]";
    }
    public interface DeltaTimed {
	int getDelta();
	void setDelta(int delta);
    }
}
