package com.kclowes.sim11;
import java.util.*;
/** Default base class for an Interrupt Request (Maskable).
 * This class is often subclassed and the <code>isActive</code> method
 * overridden.
 *
 * <p>
 * An interrupt request can be added to <code>PendingInterrupts</code>
 * at any time.  The pending interrupts are examined after the execution
 * of an instruction if the I-bit in the Condition Code register is clear.
 * Consequently, only <emph>maskable</emph> use this class. Unmaskable
 * interrupts (such as the <code>SWI</code> instruction or RESET do not
 * use this class.
 * </p>
 * @see PendingInterrupts
 * @version 1.1 (November 23, 2002) (PREV:August 20, 2002)
 * @author Ken Clowes
 */

public class InterruptRequest {

    /* Uncomment ONE of the following definitions. */
    private static boolean DEBUG = false;
//     private static boolean DEBUG = true;

    private int vector;
    private CPU6811 cpu =  CPU6811.getInstance();
    private MemorySpace mem = MemorySpace64K.getInstance();

    /** Create a Maskable InterruptRequest with the given Vector.
     */
    public InterruptRequest(int v)
    {
	vector = v;
    }


//     public void setVector(int v)
//     {
// 	vector = v;
//     }


    /** Determine if the interrupt is still active.
     * Subclasses should override this method.
     */
    protected boolean isActive()
    {
	return true;
    }

    /** Get vector for interrupt request.
     *
     * @return vector Address of vector.
     */
    public int getVector()
    {
	return vector;
    }

    private Instruction pushAll = new DummyPushAll();
    private DummyRead16 read16 = new DummyRead16();
    /** Stacks registers, sets CC I bit and sets PC to vector.
     */
    public void handle()
    {
	if (DEBUG) {
	    System.out.println("Handler vector: " +
			       Utility.int16ToHex(vector));
	}
	pushAll.execute();
	cpu.setCC_I(true);
	read16.setAddress(vector);
	int newPC = read16.execute();
	cpu.setPC(newPC);
    }

    /**
     * Returns a string representation of the interrupt request.
     * The format is: <code>InterruptRequest: vector=<hex4>@<hex4>:<boolean-is-active>
     * </code>.
     */
       
    public String toString()
    {
	int vectorContentsHi = mem.getCell(vector).get();
	int vectorContentsLo = mem.getCell(vector+1).get();
	int vectorContents = (vectorContentsHi<<8) + vectorContentsLo;
// 	System.out.println("InterruptRequest vector:" + vector +
// 			   ":" + mem.getCell(vector) +
// 			   "contents:" + vectorContentsHi);
	return "InterruptRequest: vector=" +
	    Utility.int16ToHex(vectorContents) + "@" +
	    Utility.int16ToHex(vector) +
	    ":" + isActive();
    }
}
