package com.kclowes.sim11;
/**
 * This class manages the creation and forwarding of warnings.
 *
 * @version 1.1 (July 10, 2002)
 * @author Ken Clowes
 */
public class Warn {

    private Warn() {} //Cannot instantiate this class.
    static Handler handler;

    /** The interface that clients implement to handle warnings. */
    public static interface Handler {
	/** The method invoked when a warning is created. The
	 * client (typically a user interface) can chose to wait for
	 * the user to acknowledge the warning or (as may be the case
	 * in batch applications) to simply log the warning.
	 */
	void handle(Warning w);
    }

    /** This method allows the client to specify the handler. */
    public static void setHandler(Handler h)
    {
	handler = h;
    }

    /** The basic interface for all warnings. */
    public static interface Warning {
	/** This method returns a description of the warning. */
	public String getMessage();

	/** This method returns the name of the warning type. */
	public String getName();
    }

    private static int nWarnings = 0;

    /** This method returns the total number of enabled warnings
     * that have been issued. */
    public static int getNumWarnings() {
	return nWarnings;
    }

    /** This method returns the total number of enabled warnings
     * that have been issued as a decimal String. */
    public static String getNumWarningsToString() {
	return Integer.toString(nWarnings);
    }

    /** This factory method creates a new warning when a ROM byte
     * is written to. */
    public static void ROMWrite(ROMByteCell cell)
    {
	ROMWrite warning = new ROMWrite(cell);
	nWarnings++;
	if (handler != null) {
	    handler.handle(warning);
	}
    }

    /** This class implements a Warning caused by a write to a ROM location. */
    static class ROMWrite implements Warning {
	private String msgStart = "Write to ROM at location: ";
	int address;
	private ROMWrite(ROMByteCell cell)
	{
	    address = cell.getAddress();
	}
	
	public String getMessage()
	{
	    String hexAddress = "0x" + Integer.toHexString(address);
	    return msgStart + hexAddress;
	}

	public String getName()
	{
	    return "Write to ROM";
	}
    }

    /** This factory method creates a new warning when a WOM byte
     * is read. */
    public static void WOMRead(Cell cell)
    {
	WOMRead warning = new WOMRead(cell);
	nWarnings++;
	if (handler != null) {
	    handler.handle(warning);
	}
    }

    /** This class implements a Warning caused by a read to a WOM location. */
    static class WOMRead implements Warning {
	private String msgStart = "Read from WOM (write-only memory) " +
	    "at location: ";
	int address;
	private WOMRead(Cell cell)
	{
	    address = cell.getAddress();
	}
	
	public String getMessage()
	{
	    String hexAddress = "0x" + Integer.toHexString(address);
	    return msgStart + hexAddress;
	}

	public String getName()
	{
	    return "Read from WOM";
	}
    }

    /** This factory method creates a new warning when an Illegal opcode
     * is executed. */
    public static void IllegalOpcode(int pc)
    {
	IllegalOpcode warning = new IllegalOpcode(pc);
	nWarnings++;
	if (handler != null) {
	    handler.handle(warning);
	}
    }

    /** This class implements a Warning caused by Illegal Opcode. */
    static class IllegalOpcode implements Warning {
	private String msgStart = "ILLEGAL op code encountered at location: ";
	int address;
	private IllegalOpcode(int pc)
	{
	    address = pc - 1;
	}
	
	public String getMessage()
	{
	    String hexAddress = "0x" + Integer.toHexString(address);
	    return msgStart + hexAddress;
	}

	public String getName()
	{
	    return "ILLEGAL opcode";
	}
    }

    /** This factory method creates a new warning when the TEST
     * instruction is executed. */
    public static void NoTestInstruction(int pc)
    {
	NoTestInstruction warning = new NoTestInstruction(pc);
	nWarnings++;
	if (handler != null) {
	    handler.handle(warning);
	}
    }

    /** This class implements a Warning caused by executing
     * the TEST instruction. */
    static class NoTestInstruction implements Warning {
	private String msgStart =
	    "No TEST instruction in this mode. Opcode 0x00 at: ";
	int address;
	private NoTestInstruction(int pc)
	{
	    address = pc;
	}
	
	public String getMessage()
	{
	    String hexAddress = "0x" + Integer.toHexString(address);
	    return msgStart + hexAddress;
	}

	public String getName()
	{
	    return "NoTestInstruction";
	}
    }


    /** This class implements a Warning caused by probable
     * stack underflow or overflow. */
    static class StackUnderOverFlow implements Warning {
	private String msgStartOver =
	    "Probable stack overflow.";
	private String msgStartUnder =
	    "Probable stack underflow.";
	private boolean overFlow = true;
	private String msgStart = null;
	private int oldSP;
	private int newSP;
	private StackUnderOverFlow(int oldSP, int newSP)
	{
	    this.newSP = newSP&0xffff;
	    this.oldSP = oldSP&0xffff;
	    overFlow = oldSP > newSP;
	    msgStart = (overFlow) ? msgStartOver : msgStartUnder;
	}
	
	public String getMessage()
	{
	    return msgStart +
		"  Stack Pointer changed from  0x" +
		Integer.toHexString(oldSP) +
		" to 0x" + Integer.toHexString(newSP) + ".";
	}

	public String getName()
	{
	    return "StackUnderOverFlow";
	}
    }

    /** This factory method creates a new warning for stack
     * underflow or overflow. */
    public static void StackUnderOverFlow(int prevSP, int nowSP)
    {
	StackUnderOverFlow warning = new StackUnderOverFlow(prevSP, nowSP);
	nWarnings++;
	if (handler != null) {
	    handler.handle(warning);
	}
    }


    /** This factory method creates a new Custom warning. */
     
    public static void Custom(Cell cell, String msg)
    {
	Warning warning = new Custom(cell, msg);
	nWarnings++;
	if (handler != null) {
	    handler.handle(warning);
	}
    }

    /** This class implements a Custom Warning. */
    static class Custom implements Warning {
	private String message;
	int address;
	private Custom(Cell cell, String msg)
	{
	    address = cell.getAddress();
	    message = msg;
	}
	
	public String getMessage()
	{
	    String hexAddress = " at 0x" + Integer.toHexString(address);
	    return message;// + hexAddress;
	}

	public String getName()
	{
	    return "Custom";
	}
    }

    /** This factory method creates a new warning when an Unused byte
     * is written to. */
    public static void UnusedWrite(UnusedByteCell cell)
    {
	UnusedWrite warning = new UnusedWrite(cell);
	nWarnings++;
	if (handler != null) {
	    handler.handle(warning);
	}
    }

    /** This class implements a Warning caused by a write to an Unused
     *	location.
     */
    static class UnusedWrite implements Warning {
	private String msgStart = "Write to Unmapped memory at: ";
	int address;
	private UnusedWrite(UnusedByteCell cell)
	{
	    address = cell.getAddress();
	}
	
	public String getMessage()
	{
	    String hexAddress = "0x" + Integer.toHexString(address);
	    return msgStart + hexAddress;
	}

	public String getName()
	{
	    return "Write to Unused Memory Location";
	}
    }
    /** This factory method creates a new warning when an Unused byte
     * is read. */
    public static void UnusedRead(UnusedByteCell cell)
    {
	UnusedRead warning = new UnusedRead(cell);
	nWarnings++;
	if (handler != null) {
	    handler.handle(warning);
	}
    }

    /** This class implements a Warning caused by a read of an Unused
     *	location.
     */
    static class UnusedRead implements Warning {
	private String msgStart = "Read of Unmapped memory at: ";
	int address;
	private UnusedRead(UnusedByteCell cell)
	{
	    address = cell.getAddress();
	}
	
	public String getMessage()
	{
	    String hexAddress = "0x" + Integer.toHexString(address);
	    return msgStart + hexAddress;
	}

	public String getName()
	{
	    return "Read of Unused Memory Location";
	}
    }
}
