package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the ADC Control/Status device register.
 * <p>
* The  ADCTL register combines control and status
 * bits in a single register.  The status bit is read only (it
 * is set and cleared by the simulated hardware.)
 * </p>
 *
 * <p>
 * The register is formatted as follows:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *     CCF  N/A   SCAN  MULT   CD    CC    CB    CA     Mneumonics
 *      0     0    U     U     U     U      U     U     Reset values
 *   1->ConversionComplete
 *
 * </pre>
 * </p>
 *
 *  @version 1.0 (September 21, 2002)
 *  @author Ken Clowes
 */

public class ADCControl
    extends SpecialByteCell
    implements Cell {
    

    public final static int CONVERSION_CYCLES = 128;
    private MemorySpace mem = MemorySpace64K.getInstance();
    VoltageGetter voltageGetter = null;
    boolean continuousScan = false;
    boolean multiChannel = false;
    int channel = 0;
    int command = 0;

    public ADCControl(int address)
    {
	super(address);
	set(0);
    }
    
    /** Simulates a write bus cycle to the ADCTL register. The number of times
     * the byte is written to is tracked.
     *
     * @param cmd What is written to the byte.
     */
    public void write(final int cmd)
    {
	nWrites++;
	command = cmd;
	cpu.incrementWriteCycles();
	set((get()&0x7f));
	continuousScan = ((command&0x20) == 0x20);
	multiChannel = ((command&0x10) == 0x10);
	channel = command&0x0f;
	cpu.addTimedEvent(new TimedEvent(CONVERSION_CYCLES, new ADCAction()));
    }

    final public int read() //Can't I inherit this?
    {
	nReads++;
	cpu.incrementReadCycles();
	return  get();
    }
    
    /** Get the current VoltageGetter. */
    public final VoltageGetter getVoltageGetter()
    {
	return voltageGetter;
    }

    /** Set the VoltageGetter. */
    public final void setVoltageGetter(VoltageGetter getter)
    {
	voltageGetter = getter;
    }

    /** Determines if the ADC has completed conversions.
     */
    public boolean isCompleted()
    {
	return (get() & 0x80) == 0x80;
    }
    
    /** Resets the CCF Bit.
     */
    void setCompleting()
    {
	set(get() | 0x80); //Force MSB to 1; don't change other bits.
    }

    /**
     * String representation of this ADC control register.
     * The format is: <code>ADCTL at 0x<4-hexdigits></code>.
     */
    public String toString()
    {
	return "ADCTL at 0x" + Utility.int16ToHex(address);
    }

    public String getName()
    {
	return "ADCControl/Status";
    }

    /** The action performed when a conversion is complete.
     *  Unlike the real 6811, we treat conversion as a whole
     *  for 4 channels.  (i.e. the action is performed after
     *  128 bus cycles; individual memory locations 0x1031 -- 0x1034
     *  are <strong>not</strong> updated each 32 bus cycles.
     */
    public class ADCAction implements TimedAction {

	public void doAction()
	{
	    VoltageGetter getter = getVoltageGetter();
	    ADC adc = ADC.getInstance();
	    ADCResult [] resultRegisters = adc.getResultRegisters();
	    set(0x80 + (command&0x3f));
	    int results[] = getter.getVoltages();
	    assert(results.length == 8);
	    int offset = (channel > 3) ? 4 : 0;
	    for (int i =0; i < 4; i++) {
		resultRegisters[i].set(results[i+offset]);
	    }

	    if (continuousScan) {
		cpu.addTimedEvent(new TimedEvent(CONVERSION_CYCLES,
						 new ADCAction()));
	    }
	}
    }
}

