package com.kclowes.sim11.devices;

/** Constants for internal 6811 device registers.  The 6811
 * incorporates various internal devices (such as port A, port E,
 * timer, A/D conversion, etc.) modules.  These internal devices are
 * mapped to a 64-byte memory segment (usually to the addresses
 * 0x1000--0x103F.)
 *
 * <p> This convenience class provides symbolic
 * constants for the various device registers.  The symbolic names
 * reflect Motorola's conventions as described in the "Pink Book".
 * </p>
 *
 * <p>
 * The class is currently implemented with <code>public final static</code>
 * constants.  However, the real 6811 allows the addressing range
 * to be re-mapped during the first few dozen cycles following
 * RESET.  Indeed, the real 6811 does several things during the 64 bus
 * cycles following RESET. Simulating a real RESET would require (amongst
 * other things) that the constants in this class not be <code>final</code>.
 * </p>
 *
 *  @version 1.2 (October 29, 2002)
 *  @author Ken Clowes
 */

public class DevRegisters {
    /**
     * The base address for internal 6811 device registers.
     */
    public static final int REG_BAS = 0x1000;

    /**
     * The register number (offset from REG_BAS) for the
     * free-running cycle counter.  Note that this is a 16-bit
     * register.
     */
    public static final int TCNT = 0x0e;

    /**
     * The absolute address of the hi-byte of the 
     * free-running cycle counter.
     */
    public static final int TCNT_HI_ADDRESS = REG_BAS + TCNT;

    /**
     * The absolute address of the lo-byte of the 
     * free-running cycle counter.
     */
    public static final int TCNT_LO_ADDRESS = TCNT_HI_ADDRESS + 1;

    /**
     * The register number (offset from REG_BAS) for the
     * timer flag 2 (TFLG2) register.
     */
    public static final int TFLG2 = 0x25;

    /**
     * The absolute address of the TFLG2 register.
     */
    public static final int TFLG2_ADDRESS = REG_BAS + TFLG2;

    /**
     * The register number (offset from REG_BAS) for the
     * timer mask 2 (TMSK2) register.
     */
    public static final int TMSK2 = 0x24;

    /**
     * The absolute address of the TMSK2 register.
     */
    public static final int TMSK2_ADDRESS = REG_BAS + TMSK2;

    /**
     * The register number (offset from REG_BAS) for the
     * AD Control Register (ADCTL) register.
     */
    public static final int ADCTL = 0x30;

    /**
     * The absolute address of the ADCTL register.
     */
    public static final int ADCTL_ADDRESS = REG_BAS + ADCTL;

    /**
     * The register number (offset from REG_BAS) for the
     * Output Compare 1 Mask Register (OC1M).
     */
    public static final int OC1M = 0x0C;

    /**
     * The absolute address of the OC1M register.
     */
    public static final int OC1M_ADDRESS = REG_BAS + OC1M;

    /**
     * The register number (offset from REG_BAS) for the
     * OPTION register.
     */
    public static final int OPTION = 0x39;

    /**
     * The absolute address of the OPTION register.
     */
    public static final int OPTION_ADDRESS = REG_BAS + OPTION;

    /**
     * The register number (offset from REG_BAS) for the
     * PORTA device  register.
     */
    public static final int PORTA = 0x00;

    /**
     * The absolute address of the PORTA device register.
     */
    public static final int PORTA_ADDRESS = REG_BAS + PORTA;

    /**
     * The register number (offset from REG_BAS) for the
     * BPROT device  register.
     */
    public static final int BPROT = 0x35;

    /**
     * The absolute address of the BPROT device register.
     */
    public static final int BPROT_ADDRESS = REG_BAS + BPROT;


    /**
     * The register number (offset from REG_BAS) for the
     * TCTL1 device  register.
     */
    public static final int TCTL1 = 0x20;

    /**
     * The absolute address of the TCTL1 device register.
     */
    public static final int TCTL1_ADDRESS = REG_BAS + TCTL1;

    /**
     * The register number (offset from REG_BAS) for the
     * BAUD device  register.
     */
    public static final int BAUD = 0x2B;

    /**
     * The absolute address of the BAUD device register.
     */
    public static final int BAUD_ADDRESS = REG_BAS + BAUD;


    /**
     * The register number (offset from REG_BAS) for the
     * SCCR1 device  register.
     */
    public static final int SCCR1 = 0x2C;

    /**
     * The absolute address of the SCCR1 device register.
     */
    public static final int SCCR1_ADDRESS = REG_BAS + SCCR1;

    /**
     * The register number (offset from REG_BAS) for the
     * SCCR1 device  register.
     */
    public static final int SCCR2 = 0x2D;

    /**
     * The absolute address of the SCCR2 device register.
     */
    public static final int SCCR2_ADDRESS = REG_BAS + SCCR2;

    /**
     * The register number (offset from REG_BAS) for the
     * SCSR device  register.
     */
    public static final int SCSR = 0x2e;

    /**
     * The absolute address of the SCSR device register.
     */
    public static final int SCSR_ADDRESS = REG_BAS + SCSR;

    /**
     * The register number (offset from REG_BAS) for the
     * SCDR device  register.
     */
    public static final int SCDR = 0x2f;

    /**
     * The absolute address of the SCDR device register.
     */
    public static final int SCDR_ADDRESS = REG_BAS + SCDR;

    /**
     * The register number (offset from REG_BAS) for the
     * COPRST device  register.
     */
    public static final int COPRST = 0x3a;

    /**
     * The absolute address of the COPRST device register.
     */
    public static final int COPRST_ADDRESS = REG_BAS + COPRST;

    /**
     * The register number (offset from REG_BAS) for the
     * TOC1 device  register.
     */
    public static final int TOC1 = 0x16;

    /**
     * The absolute address of the TOC1 device register.
     */
    public static final int TOC1_ADDRESS = REG_BAS + TOC1;

    /**
     * The register number (offset from REG_BAS) for the
     * TOC2 device  register.
     */
    public static final int TOC2 = 0x18;

    /**
     * The absolute address of the TOC2 device register.
     */
    public static final int TOC2_ADDRESS = REG_BAS + TOC2;

    /**
     * The register number (offset from REG_BAS) for the
     * TOC3 device  register.
     */
    public static final int TOC3 = 0x1A;

    /**
     * The absolute address of the TOC3 device register.
     */
    public static final int TOC3_ADDRESS = REG_BAS + TOC3;

    /**
     * The register number (offset from REG_BAS) for the
     * TOC4 device  register.
     */
    public static final int TOC4 = 0x1C;

    /**
     * The absolute address of the TOC4 device register.
     */
    public static final int TOC4_ADDRESS = REG_BAS + TOC4;

    /**
     * The register number (offset from REG_BAS) for the
     * TOC5 device  register.
     */
    public static final int TOC5 = 0x1E;

    /**
     * The absolute address of the TOC5 device register.
     */
    public static final int TOC5_ADDRESS = REG_BAS + TOC5;

    /**
     * The register number (offset from REG_BAS) for the
     * TFLG1 device  register.
     */
    public static final int TFLG1 = 0x23;

    /**
     * The absolute address of the TFLG1 device register.
     */
    public static final int TFLG1_ADDRESS = REG_BAS + TFLG1;

    /**
     * The register number (offset from REG_BAS) for the
     * CFORC device  register.
     */
    public static final int CFORC = 0x0b;

    /**
     * The absolute address of the CFORC device register.
     */
    public static final int CFORC_ADDRESS = REG_BAS + CFORC;

    /**
     * The register number (offset from REG_BAS) for the
     * PORTE device  register.
     */
    public static final int PORTE = 0x0A;

    /**
     * The absolute address of the PORTE device register.
     */
    public static final int PORTE_ADDRESS = REG_BAS + PORTE;

     /**
     * The register number (offset from REG_BAS) for the
     * PCTLA device  register.
     */
    public static final int PCTLA = 0x26;

 
    /**
     * The absolute address of the PCTLA device register.
     */
    public static final int PCTLA_ADDRESS = REG_BAS + PCTLA;

 
    /**
     * This class cannot be instantiated; it contains only static constants.
     */
    private DevRegisters(){} 

}

