package com.kclowes.sim11.devices;
import junit.framework.*;
import com.kclowes.sim11.*;
import java.io.*;

/**
 * Some simple tests.
 *
 *  @version 1.4 (October 28, 2002)
 *  @author Ken Clowes
 *
 */

public class DevicesTest extends TestCase {

    static private final boolean DEBUG = false;
    //static private final boolean DEBUG = true;

    private MemorySpace mem = MemorySpace64K.getInstance();
    private CPU6811 cpu = CPU6811.getInstance();
    private ADCControl adcControl = null;
    private ADC adc = null;
    public DevicesTest(String name) {
	super(name);
    }
    
    protected void setUp() {
	cpu.removeALLTimedEvents();
  	new com.kclowes.sim11.devices.Timer();
	adc = ADC.getInstance();
	adcControl = adc.getAdcControl();
    }
    
    public static Test suite() {
	
	TestSuite suite= new TestSuite();
    	suite.addTest(new DevicesTest("testTimer1"));
    	suite.addTest(new DevicesTest("testTimer2"));
    	suite.addTest(new DevicesTest("testIsSpecialByte"));
   	suite.addTest(new DevicesTest("testLoadLCD"));
  	suite.addTest(new DevicesTest("testBasicADC"));
  	suite.addTest(new DevicesTest("testSCITransmit"));
	suite.addTest(new DevicesTest("testTOC1"));
	suite.addTest(new DevicesTest("testPortE"));
	return suite;
    }
    
    /** Basic test of Timer Overflow.
     * <p>
     * We verify that the TOF flag bit (bit 7 of the TFLG2 device register)
     * is initally clear and reamins clear until 0x10000 clock cycles have
     * elapsed.
     * </p>
     */
    public void testTimer1()
    {
	Cell tflg2Cell = mem.getCell(0x1025);
	assertEquals(0, tflg2Cell.get()&0x80);
	for(int i = 0; i < 0xffff; i++) {
	    cpu.incrementCycles();
	    assertEquals(0, tflg2Cell.get()&0x80);
	}
	assertEquals(0, tflg2Cell.get()&0x80);
	cpu.incrementCycles();
	assertEquals(0x80, tflg2Cell.get());
    }
    /** More tests of Timer Overflow.
     * <p>
     * We verify that the TOF flag bit (bit 7 of the TFLG2 device register)
     * is initally clear and reamins clear until 0x10000 clock cycles have
     * elapsed. It is reset by a write of 1 to TFLG2.
     * </p>
     */
    public void testTimer2()
    {
	Cell tflg2Cell = mem.getCell(0x1025);
	assertEquals(0, tflg2Cell.get()&0x80);
	for(int i = 0; i < 0xffff; i++) {
	    cpu.incrementCycles();
	    assertEquals(0, tflg2Cell.get()&0x80);
	}
	assertEquals(0, tflg2Cell.get()&0x80);
	cpu.incrementCycles();
	assertEquals(0x80, tflg2Cell.get());
	
	cpu.incrementCycles();
	assertEquals(0x80, tflg2Cell.get());
	tflg2Cell.write(0x80);
	assertEquals(0x00, tflg2Cell.get());
	for(int i = 0; i < 0xfffd; i++) {
	    cpu.incrementCycles();
	    assertEquals(0, tflg2Cell.get()&0x80);
	}
	assertEquals(0, tflg2Cell.get()&0x80);
	cpu.incrementCycles();
	assertEquals(0x80, tflg2Cell.get()&0x80);
	
	cpu.incrementCycles();
	assertEquals(0x80, tflg2Cell.get());
	tflg2Cell.write(0x80);
	assertEquals(0x00, tflg2Cell.get());
	for(int i = 0; i < 0xfffd; i++) {
	    cpu.incrementCycles();
	    assertEquals(0, tflg2Cell.get()&0x80);
	}
	assertEquals(0, tflg2Cell.get()&0x80);
	cpu.incrementCycles();
	assertEquals(0x80, tflg2Cell.get()&0x80);
    }
    /** Test of Timer configuration.  Frankly, I forget why this
     * test exists.  But hey...I'm not going to remove it!
     */
    public void testIsSpecialByte()
    {
	Cell cell;
	cell = mem.getCell(DevRegisters.TFLG2_ADDRESS);
	assertEquals(0, cell.get()&0x80);
	assertTrue(cell instanceof SpecialByteCell);
	cell = mem.getCell(DevRegisters.TCNT_HI_ADDRESS);
	assertTrue(cell instanceof SpecialByteCell);
	cell = mem.getCell(DevRegisters.TCNT_LO_ADDRESS);
	assertTrue(cell instanceof SpecialByteCell);
	cell = mem.getCell(DevRegisters.TMSK2_ADDRESS);
	assertTrue(cell instanceof SpecialByteCell);
    }

    private static final int HOME_CURSOR = 0x02;

    /** Verify that the LCD module can be loaded and works.
     */
    public void testLoadLCD()
    {
	Warn.setHandler(new Warn.Handler() {
        public void handle(Warn.Warning w)
        {
            System.err.println("Warning: " + w.getMessage());
        }
    });
    
	int nWarns = Warn.getNumWarnings();
	LCDControl cellLCDControl = (LCDControl)
	    mem.getCell(LCD.LCD_BASE_ADDRESS);
	Cell cellLCDData = mem.getCell(LCD.LCD_BASE_ADDRESS + 1);
	LCD lcd = LCD.getInstance();
	cellLCDControl.addCellChangeListener(new CellChangeListener() {
		public void perform(Cell c)
		{
		    ;
		}
	    });
	assertTrue(cellLCDControl instanceof SpecialByteCell);
	assertTrue(cellLCDData instanceof SpecialByteCell);
	assertEquals(0, cellLCDControl.get());//initially not busy
	cellLCDControl.write(0x01);//Clear display command
	assertEquals(0x80, cellLCDControl.get()&0x80);//Busy after write
	assertEquals(0, Warn.getNumWarnings() - nWarns); //No Warnings yet
	cellLCDControl.write(0x01);//Clear display command but BUSY
	assertEquals(1, Warn.getNumWarnings() - nWarns); //Should have warning
	for(int i = 0; i < 1000; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x00, cellLCDControl.get()&0x80);//Not busy after wait
	cellLCDData.write('A');//Write a character
	assertEquals(1, Warn.getNumWarnings() - nWarns); //Should have warning
	assertEquals(0x80, cellLCDControl.get()&0x80);//Busy after write
	cellLCDData.write('B');//Write a character
	assertEquals(2, Warn.getNumWarnings() - nWarns); //Should have warning
	for(int i = 0; i < 1000; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x00, cellLCDControl.get()&0x80);//Not busy after wait
	char lines[][];
	lines = cellLCDControl.getLines();
	assertEquals('A', lines[0][0]);
	assertEquals(' ', lines[1][0]);
	cellLCDData.write('B');//Write a character
	assertEquals('B', lines[0][1]);
	for(int i = 0; i < 1000; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x00, cellLCDControl.get()&0x80);//Not busy after wait

	cellLCDControl.write(HOME_CURSOR);
	for(int i = 0; i < 1000; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x00, cellLCDControl.get()&0x80);//Not busy after wait
	cellLCDData.write('X');//Write a character
	lines = cellLCDControl.getLines();
	assertEquals('X', lines[0][0]);
	assertEquals(0x80, cellLCDControl.get()&0x80);//Busy
	for(int i = 0; i < 1000; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x00, cellLCDControl.get()&0x80);//Not busy after wait
	cellLCDControl.write(0x80 + 5); //move cursor to column 5 upper row
	for(int i = 0; i < 1000; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x00, cellLCDControl.get()&0x80);//Not busy after wait
	cellLCDData.write('6');//Write a character
	lines = cellLCDControl.getLines();
	assertEquals('6', lines[0][5]);

    }

    /** Verify that the ADC can be loaded.
     */
    public void testBasicADC()
    {
	Warn.setHandler(new Warn.Handler() {
		public void handle(Warn.Warning w)
		{
		    System.err.println("Warning: " + w.getMessage());
		}
	    });
		    
	int nWarns = Warn.getNumWarnings();
	assertEquals(adcControl.getVoltageGetter(), null);
	adcControl.setVoltageGetter(null);
	assertEquals(adcControl.getVoltageGetter(), null);
	VoltageGetter myVoltageGetter = new VoltageGetter()
	    { public int[] getVoltages()
		{
		    int [] voltages = new int[8];
		    for(int i = 0; i <= 7; i++) {
			voltages[i] = i*32;
		    }
		    return voltages;
		}
	    };
	
	adcControl.setVoltageGetter(myVoltageGetter);
	assertEquals(adcControl.getVoltageGetter(), myVoltageGetter);
	assertTrue(adcControl instanceof SpecialByteCell);
	assertTrue(!((Cell) adcControl instanceof RAMByteCell));
	assertTrue(!adcControl.isCompleted());
	adcControl.write(0x00); //Convert AN0--AN3 one time
	assertTrue(!adcControl.isCompleted());
	for(int i = 0; i < ADCControl.CONVERSION_CYCLES-1; i++) {
	    cpu.incrementCycles();
	}
	assertTrue(!adcControl.isCompleted());
	cpu.incrementCycles();
	assertTrue(adcControl.isCompleted());
	assertEquals(0x80, adcControl.get());
	ADCResult result_1 = (ADCResult)
	    mem.getCell(DevRegisters.ADCTL_ADDRESS+1);
	ADCResult result_2 = (ADCResult)
	    mem.getCell(DevRegisters.ADCTL_ADDRESS+2);
	ADCResult result_3 = (ADCResult)
	    mem.getCell(DevRegisters.ADCTL_ADDRESS+3);
	ADCResult result_4 = (ADCResult)
	    mem.getCell(DevRegisters.ADCTL_ADDRESS+4);
	assertTrue(result_1 instanceof ADCResult);
	assertEquals(0, result_1.get());
	assertEquals(32, result_2.get());
	assertEquals(64, result_3.get());
	assertEquals(96, result_4.get());


	adcControl.write(0x04); //Convert AN4--AN7 one time
	assertTrue(!adcControl.isCompleted());
	for(int i = 0; i < ADCControl.CONVERSION_CYCLES-1; i++) {
	    cpu.incrementCycles();
	}
	assertTrue(!adcControl.isCompleted());
	cpu.incrementCycles();
	assertTrue(adcControl.isCompleted());
	assertEquals(128, result_1.get());
	assertEquals(160, result_2.get());
	assertEquals(192, result_3.get());
	assertEquals(224, result_4.get());
    }

    /** Verify that the SCI transmitter works.
     */
    public void testSCITransmit()
    {
	Scsr scsr = Scsr.getInstance();
	Scdr scdr = Scdr.getInstance();
	scsr.reset();
	scdr.reset();
	assertTrue(scsr.isTxEmpty());
	assertEquals(scdr.getTxBuffer(), 0);
	scdr.write('H');
	assertEquals(scdr.getTxBuffer(), 'H');
	assertTrue(!scsr.isTxEmpty());
	for(int i = 0; i < 1000; i++) {
	    cpu.incrementCycles();
	}
	assertTrue(scsr.isTxEmpty());
	assertEquals(scdr.getTxBuffer(), 'H');
	assertTrue(!scsr.isRxFull());
	scdr.setRxBuffer('x');
	assertTrue(scsr.isRxFull());
	assertEquals(scdr.read(), 'x');
	assertTrue(!scsr.isRxFull());
	
   }

    /** Verify that TOC1 register works.
     */
    public void testTOC1()
    {
	PortA porta = PortA.getInstance();
	TOC1lo toc1Lo = TOC1lo.getInstance();
	TOC5hi toc5Hi = TOC5hi.getInstance();
	TOC5lo toc5Lo = TOC5lo.getInstance();
	assertEquals(toc1Lo.get(), 0xff);

	//!!Assumes that all OCs have already fired from previous tests.
	Tflg1 tflg1 = Tflg1.getInstance();
	assertEquals(tflg1.get(), 0xF8);

	//Clear TOC5 Flag
	tflg1.write(0x08);
	assertEquals(tflg1.get(), 0xF0);

	while (cpu.getDeltaList().size() > 1) {
	    cpu.incrementCycles();
	}
	if (DEBUG) {
	    System.out.println("Cycles: " +
			       Utility.int16ToHex((int)cpu.getNCycles()) +
			       " Pending events: " +
			       cpu.getDeltaList().size());
	}
	for(int i = 0; i <0x10000; i++) {
	    cpu.incrementCycles();
	}
	if (DEBUG) {
	    System.out.println("Cycles: " +
			       Utility.int16ToHex((int)cpu.getNCycles()) +
			       " Pending events: " +
			       cpu.getDeltaList().size());
	}

	//Set TOC5 next time out in 100 (0x64) cycles

	long t = (cpu.getNCycles()+100)&0xffff;
	toc5Hi.write((int)(t>>8)&0xff);
	toc5Lo.write((int)(t&0xFF));
	assertEquals(2, cpu.getDeltaList().size());
	assertEquals(0x00, tflg1.get()&0x08); //TOC5Flag should be clear
	for(int i = 0; i < 98; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x00, tflg1.get()&0x08); //TOC5Flag should be clear
	cpu.incrementCycles();
	assertEquals(0x08, tflg1.get()&0x08); //TOC5Flag should be set
	//Clear TOC5Flag
	tflg1.write(0x08);
	assertEquals(tflg1.get()&0x08, 0x00);
	
	//Clear PortA
	porta.write(0x00);
	assertEquals(0x00, porta.get());

	//Setup TCTL1 so that next TOC5 event sets PortA_bit
	Tctl1 tctl1 = Tctl1.getInstance();
	tctl1.set(0x03);

	//Configure TOC5 to match after 2000 bus cycles.
	
	t = (cpu.getNCycles()+2000)&0xffff;
	toc5Hi.write((int)(t>>8)&0xff);
	toc5Lo.write((int)(t&0xFF));
	assertEquals(2, cpu.getDeltaList().size());
	assertEquals(0x00, tflg1.get()&0x08); //TOC5Flag should be clear
	for(int i = 0; i < 2000; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x08, tflg1.get()&0x08); //TOC5Flag should be set
	assertEquals(0x08, porta.get());
	//Clear TOC5Flag
	tflg1.write(0x08);
	assertEquals(tflg1.get()&0x08, 0x00);

	//Configure TOC5 to toggle after 100 cycles
	t = (cpu.getNCycles()+100)&0xffff;
	toc5Hi.write((int)(t>>8)&0xff);
	toc5Lo.write((int)(t&0xFF));
	tctl1.set(0x01); //Toggle on TOC5
	assertEquals(2, cpu.getDeltaList().size());
	assertEquals(0x00, tflg1.get()&0x08); //TOC5Flag should be clear
	for(int i = 0; i < 100; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x08, tflg1.get()&0x08); //TOC5Flag should be set
	assertEquals(0x00, porta.get()&0x08); //PortA bit 3 should be clear
	//Clear TOC5Flag
	tflg1.write(0x08);
	assertEquals(tflg1.get()&0x08, 0x00);

	//Toggle TOC5 again
	t = (cpu.getNCycles()+100)&0xffff;
	toc5Hi.write((int)(t>>8)&0xff);
	toc5Lo.write((int)(t&0xFF));
	//tctl1.set(0x01); //Toggle on TOC5
	assertEquals(2, cpu.getDeltaList().size());
	assertEquals(0x00, tflg1.get()&0x08); //TOC5Flag should be clear
	for(int i = 0; i < 100; i++) {
	    cpu.incrementCycles();
	}
	assertEquals(0x08, tflg1.get()&0x08); //TOC5Flag should be set
	assertEquals(0x08, porta.get()&0x08); //PortA bit 3 should be set
	//Clear TOC5Flag
	tflg1.write(0x08);
	assertEquals(tflg1.get()&0x08, 0x00);
    }
    /** Verify that PortE register works.
     */
    public void testPortE()
    {
	int voltages[] = {0,201,2,3,4,5,6,0xff};
	PortE portE = PortE.getInstance();
	int nWarns = Warn.getNumWarnings();
	portE.write(0);
	assertEquals(nWarns+1, Warn.getNumWarnings());
	portE.setVoltages(voltages);
	int v;
	v = portE.read();
	assertEquals(0x82, v);
	voltages[0] = 123;
	voltages = portE.getVoltages();
	assertEquals(0, voltages[0]);
	
	
    }

}
