package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the LCD Control/Status device register.
 * <p>
 * The LCD_CONTROL register combines control and status
 * bits in a single register.  The status bit is read only (it
 * is set and cleared by the simulated hardware.) This device
 * register interacts strongly with the assoicated LCD data register
 * {@link LCDData LCDData}.  This class is used by {@link LCD LCD}.
 * </p>
 *
 * <p>
 * The register is formatted as follows:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *     BUSY                                             Mneumonics
 *      0                                               Reset values
 *   1->BUSY
 * </pre>
 * </p>
 *
 * <p>
 *  Commands can be written to the register.  The commands are:
 * <ol>
 *  <li>Clear the display: 0x01</li>
 *  <li>Home cursor: 0x02</li>
 *  <li>Cursor control: %1xxxxxxx</li>
 * </ol>
 * </p>
 *
 * <p>
 * This is NOT a singleton!  There are reasons for this
 * choice (e.g. more than one LCD display device). 
 * </p>
 *  @version 1.1 (October 16, 2002)
 *  @author Ken Clowes
 */

public class LCDControl
    extends SpecialByteCell
    implements Cell {
    

    private int BUSY_NUM_CYCLES = 1000;
    private char[][] lines= new char[2][20];
    private int row = 0;
    private int col = 0;

    private MemorySpace mem = MemorySpace64K.getInstance();
    
    public LCDControl(int address)
    {
	super(address);
	clear();
	set(0);
    }
    
    void clear()
    {
	for(int i = 0; i <20 ; i++) {
	    lines[0][i] = ' ';
	    lines[1][i] = ' ';
	}
	col = 0;
	row = 0;
    }

    final void addCharacter(char ch)
    {
	if (row >= 2) {
	    Warn.Custom(this,
			"Write to LCD Data register (" +
			Utility.int16ToHex(address) +
			") ignored; DISPLAY IS FULL.");
	} else {
	    lines[row][col] = ch;
	    col++;
	    if (col == 20) {
		col = 0;
		row++;
	    }
	}
    }
    
    final public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  get();
    }
    
    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked.
     *
     * @param command What is written to the byte.
     */
    public void write(int command)
    {
	
	nWrites++;
	cpu.incrementWriteCycles();
	if (isBusy()) {
	    Warn.Custom(this,
			"Write to LCD Control register (" +
			Utility.int16ToHex(address) +
			") ignored because device busy.");
	} else {
	    setBusy();
	    if ((command & 0x80) == 0) { //Ordinary command
		switch (command) {
		case 0x01:
		    clear();
		    break;
		case 0x02:
		    col = 0;
		    row = 0;
		    break;
		default:
		    System.err.println(
				"Unknown command (0x" +
				Utility.int8ToHex(command) +
				") to LCD Control register (" +
				Utility.int16ToHex(address) +
				") ignored.");
		}
	    } else { //Cursor control command
		int position = command & 0x7f; //Get character address
		if (position < 19) {//Move cursor to upper line
		    row = 0;
		    col = position;
		} else if ((position >= 64) && (position < 84)) {//lower line
		    row = 1;
		    col = position - 64;
		} else { //Invalid cursor position
		    Warn.Custom(this,
				"Invalid cursor postion command (0x" +
				Utility.int8ToHex(command) +
				") to LCD Control register (" +
				Utility.int16ToHex(address) +
				") ignored.");
		}
	    }
	}
    }
    
    /** Determines if the LCD device is busy.
     */
    public boolean isBusy()
    {
	return (get() & 0x80) == 0x80;
    }
    
    /** Sets the Busy Bit. Schedules an event to clear the bit
     * later on.
     */
    void setBusy()
    {
	set(get() | 0x80);
	cpu.addTimedEvent(new TimedEvent(BUSY_NUM_CYCLES, new TimedAction() {
		public void doAction()
		{
		    set(get()&0x7f);
		}
	    }));
    }

    /** Get the 2-d array of lines.
     *
     */
    public char[][] getLines()
    {
	return lines;
    }

    /**
     * String representation of this LCD control register.
     * The format is: <code>LCDControl at 0x<4-hexdigits></code>.
     */
    public String toString()
    {
	return "LCDControl at 0x" + Utility.int16ToHex(address);
    }

    public String getName()
    {
	return "LCDControl/Status";
    }

    public void reset()
    {
	clear();
	set(0);
    }
}

