package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;
import java.util.*;

/** Implements PORTE. Port E is an input-only port; each pin can be
 *  set to an analog voltage.  The voltages are used directly by the
 *  Analog-to-Digital subsystem.
 *
 * <p>
 * A write to this cell produces a Warning.
 * </p>
 *
 * <p>
 *   Clients can get/set the array of voltages. In addition, they can
 *   add a <code>VoltageChangeListener</code> that will be invoked when
 *   the voltages have been set and at least one of them changes.
 * </p>
 * <p>When the <code>get()</code> or <code>read()</code> methods are
 * invoked, the underlying analog voltages are interpreted as bits
 * using the rule:
 * <pre>
 *    analog voltage < THRESHOLD   ==> 0
 *    analog voltage >= THRESHOLD  ==> 1
 *
 *        where THRESHOLD is hardcoded as 0xB3
 *            i.e. ((256/5)*35)/10
 *              where we assume a range of 0--5v
 *              and that 3.5 volts is the threshold.
 * </pre>
 * </p>
 *
 *  @see ADC
 *  @version 1.0 (November 12, 2002)
 *  @author Ken Clowes
 */

public class PortE extends SpecialByteCell implements VoltageGetter {
    static final boolean DEBUG = false;
    public final static int THRESHOLD = ((256/5)*35)/10; //0xB3
    private MemorySpace mem = MemorySpace64K.getInstance();
    private static PortE INSTANCE = null;
    public ArrayList listeners = new ArrayList();
    private int[] voltages = {0, 0, 0, 0, 0, 0, 0, 0};
    public final static PortE getInstance() {
	if (INSTANCE == null) {
	    INSTANCE = new PortE(DevRegisters.PORTE_ADDRESS);
	}
	return INSTANCE;
    }

    private PortE(int address)
    {
	super(address);
	mem.setCell(address, this);
    }

    public String getName()
    {
	return "PORTE";
    }
    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {
	nWrites++;
	cpu.incrementWriteCycles();
	Warn.Custom(this,
		    "Cannot write to Port E");

    }

    /** Set the analog voltages at each pin PortE[0--7].
     *
     * @param v Array of precisely 8 values.
     */
    public void setVoltages(int [] v)
    {
	assert (v.length == 8);
	//Copy them...
	boolean change = false;
	if (DEBUG) {
	    System.out.println("PortE setVoltages, voltages[2]: " +
			       voltages[2] +
			       " v[2]: " + v[2]);
	}
	for (int i = 0; i < 8; i++) {
	    if (voltages[i] != v[i]) {
		change = true;
		voltages[i] = v[i];
	    }
	}

        if (change) { //Invoke listeners if any voltage has changed
	    if (DEBUG) {
		System.out.println("Invoking PortE listeners");
	    }
	    if (listeners.size() != 0) {
		for(Iterator iter = listeners.iterator();
		    iter.hasNext();) {
		    ((VoltageChangeListener) iter.next()).perform(this);
		}
	    }
	}


	//Convert voltages to bits...
	int binary = 0;
	for (int i = 0, pat = 1; i < 8; i++, pat = pat<<1) {
	    if (voltages[i] >= THRESHOLD) {
		binary |= pat;
	    }
	}
	set(binary); //This will wake up all listeners
    }

    /** Returns array of analog voltages.
     */
    public int[] getVoltages()
    {
	int v[];
	v = new int[8];
	for(int i = 0; i < 8; i++) {
	    v[i] = voltages[i];
	}
	return v;
    }

    /** Adds a VoltageChangeListerer to the cell. */
    public void addVoltageChangeListener(VoltageChangeListener l)
    {
	listeners.add(l);
	if (DEBUG) {
	    System.out.println("added VoltageChangeListener: " +
			       l);
	}
    }
    
    /** Remove a VoltageChangeListener from the Cell. */
    public boolean removeVoltageChangeListener(VoltageChangeListener l)
    {
	boolean removed = false;
	for(Iterator iter = listeners.iterator();
	    iter.hasNext();) {
	    if (iter.next() == l) {
		iter.remove();
		removed = true;
		break;
	    }
	}
	return removed;
    }
}
