package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;
/**
 * This class implements the internal TCTL1 device register.
 *<p>
 * The bits in the TCTL1 register are:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *     OM2  OL2    OM3   OL3   OM4   OL4   OM5   OL5    Mneumonics
 *      0     0      0          0     0     0     0     Reset values
 * </pre>
 * </p>
 *
 * <p>
 *    Each 2-bit (no pun intended :-)!) group specifies the action
 *    taken when the corresponding Output Compare fires.  The possible
 *    actions are:
 *      <ul>
 *         <li> 00 -- do nothing</li>
 *         <li> 01 -- Toggle PortA bit</li>
 *         <li> 10 -- Clear PortA bit</li>
 *         <li> 11 -- Set PortA bit</li>
 *      </ul>
 * </p>
 *
 * @version 1.0 (October 26, 2002)
 * @author Ken Clowes
*/

public class Tctl1 extends SpecialByteCell
{

    static final int DO_NOTHING = 0;
    static final int TOGGLE     = 1;
    static final int CLEAR      = 2;
    static final int SET        = 3;
    
    private MemorySpace mem = MemorySpace64K.getInstance();
    private final static int DEFAULT_CONTENTS = 0x00;
    public static Tctl1 INSTANCE = new Tctl1(DevRegisters.TCTL1_ADDRESS,
					DEFAULT_CONTENTS);
    static public Tctl1 getInstance() {return INSTANCE;}

    private Tctl1(int address, int contents)
    {
	super(address, contents);
	mem.setCell(DevRegisters.TCTL1_ADDRESS, this);
    }

    /** Determine what should be done with PortA when an TOC event
     * occurs.
     */
    public int getPortAAction(int channel)
    {
	assert((channel >= 1) && (channel <= 5));
	switch (channel) {
	case 1:
	    return DO_NOTHING;
	case 2:
	    return (get()>>6)&0x03;
	case 3:
	    return (get()>>4)&0x03;
	case 4:
	    return (get()>>2)&0x03;
	case 5:
	    return get()&0x03;
	}
	assert(false);
	return 0; //NOT REACHED
    }

    /** Returns a string representation of a Tctl1.

     */
    public String toString()
    {
	return "TCTL1@" +
	    Utility.int16ToHex(address) + ":" +
	    Utility.int8ToHex(contents);
    }
    
    public String getName()
    {
	return "TCTL1";
    }

    public void reset()
    {
	set(DEFAULT_CONTENTS);
    }
}
