package com.kclowes.sim11.devices;
import com.kclowes.sim11.*;

/** Implements the Timer Flags 2 (TFLG2) device register.
 * <p>
 * The bits in the TFLG2 register are:
 * <pre>
 *   +-----------------------------------------------+
 *   |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |  Bit numbers
 *   +-----------------------------------------------+
 *     TOF                                              Mneumonics
 *      0                                               Reset values
 * </pre>
 *
 * <p>
 *  The TOF (Free-running counter overflow) bit
 * </p>
 *  @version 1.1 (August 27, 2002)
 *  @author Ken Clowes
 */

public class Tflg2
    extends SpecialByteCell
    implements Cell {

    private int tflg2_contents = 0;
    private PendingInterrupts pending = PendingInterrupts.getInstance();
    private Tmsk2 tmsk2 = Tmsk2.getInstance();
    final static int TOF_VECTOR = 0xffde;
    public Tflg2(int address)
    {
	super(address);
    }

    public int get()
    {
	return tflg2_contents;
    }

    public int read()
    {
	nReads++;
	cpu.incrementReadCycles();
	return  get();
    }

    public void set(int v)
    {
	tflg2_contents = v;
 	if (((v&0x80) == 0x80) && ((tmsk2.get()&0x80) == 0x80)) {
	    InterruptRequest request = new InterruptRequest(TOF_VECTOR) {
		public boolean isActive() {
		    return (((tflg2_contents&0x80)==0x80) &&
			((tmsk2.get()&0x80) == 0x80));
		}
	    };
	    pending.add(request);
 	}
    }

    /** Simulates a write bus cycle to the byte. The number of times
     * the byte is written to is tracked. The '1' bits in the contents
     * that are written clear the corresponding bits in the TFLG2
     * device register.
     *
     * @param contents What is written to the byte.
     */
    public void write(int contents)
    {

	int c;
	nWrites++;
	cpu.incrementWriteCycles();
	c = get();
	c = c & (~contents);
	set(c);
    }
    
    public String getName()
    {
	return "TFLG2";
    }
}

