package com.kclowes.sim11.ui;
import com.kclowes.sim11.*;
import com.kclowes.sim11.devices.*;
import java.util.*;
import java.io.*;
import java.util.regex.*;

/** Abstract base class for user interaction with simulator.
 * The simulator engine contains no code that interacts with the user.
 * To interact with the engine, the user has to specify system configuration,
 * send commands to the engine and so on.  Some of the common interactions
 * are specified and partially implemented in this abstract class.
 *
 * This class must be subclassed for a specific kind of user interaction.
 * Some possibilites include:
 * <ul>
 *  <li> A command line interface</li>
 *  <li> A GUI</li>
 *  <li> A remote client/server interaction between user and simulator.</li>
 *  <li> A command line interface that simulates the BUFFALO monitor.</li>
 *  <li> A command line interface designed for batch testing
 *  (including syntax for assertions).</li>
 * </ul>
 *
 *  @version 1.6 (November 11, 2003)
 *  @author Ken Clowes
 */
  
public abstract class AbstractUI
{

    //Static package variables
    /** Default starting addresses for memory dumps. */
    static final int DEFAULT_MEM_DUMPS[] = {0x7000,
					    0x003A};

    //S19 file for short Buffalo monitor
    private final String shortBuffalo_s19 =
"S12300C47EE3A67EE3A67EE3A67EE3A67EE3A67EE3A67EE3A67EE3A67EE3A67EE3A67EE371\n"+
"S11F00E4A67EE3A67EE3A67EE3A67EE3A67EE3A67EE3A67EE3A67EE3A67EE3A617\n"+
"S10AE3A6865006CF7EE3A6BA\n"+
"S123FFD600C400C700CA00CD00D000D300D600D900DC00DF00E200E500E800EB00EE00F15F\n"+
"S10DFFF600F400F700FA00FDE0003B\n"+
"S9030000FC\n";


    //Package variables
    CPU6811 cpu = CPU6811.getInstance();
    LstLoader lstLoader = LstLoader.getInstance();
    MemorySpace mem = MemorySpace64K.getInstance();


    //Private variables
    private String versionInfo =
	"sim11  -- a 68hc11 simulator -- " + Version.get() + "\n" +
	"(Based on engine com.kclowes.sim11 " +
	com.kclowes.sim11.Version.get() + "\n" +
	"and on devices com.kclowes.sim11.devices " +
	com.kclowes.sim11.devices.Version.get() + "\n\n" +
	"Copyright (C) 2002 Ken Clowes(kclowes@ee.ryerson.ca).\n\n" +
	"            ALL RIGHTS RESERVED.\n\n" +
        "Can be copied/modified/distributed under the terms of the GPL\n" +
	"(and, perhaps, other less restrictive licenses).";

    private String configFile = null; //Should implement this!

    //Package methods...

    /** Configure the memory map.
     *
     * This method is a STUB.  The standard Ryerson configuration
     * is hard coded.
     */
    void setConfig()
    {
	configFile = null;
	if (configFile == null) { //Do Ryerson Standard Configuration
	    cpu.setStopOnSWI(true);
	    configureRAM(0x6000, 0x7fff);
	    configureRAM(0, 0xff);
	    configureROM(0xe000, 0xffff);
	    
	    // Configure ROM for SWI instuction
	    int SWI_VECTOR = 0xFFF6;
	    Cell swiHi = mem.getCell(SWI_VECTOR);
	    Cell swiLo = mem.getCell(SWI_VECTOR+1);
	    swiHi.set(0xE0);
	    swiLo.set(0x00);
	    Cell cellIsr = mem.getCell(0xE000);
	    cellIsr.set(0xCF); //Stop instruction
	    

            //Load short Buffalo monitor ROM/RAM vectors
	    try {
		loadS19(new StringReader(shortBuffalo_s19));
	    } catch (IOException e) {
		System.out.println(
				   "Got unexpected exception reading String:"
				   + e);
		System.exit(-1);
	    }
	    //DO devices
	    new com.kclowes.sim11.devices.Timer();
	    LCD lcd = LCD.getInstance(); //KLUDGE
	    ADC adc = ADC.getInstance();
	    mem.setCell(0x1100, new RdWrByteCell(0x1100));
	    PortA.getInstance();
	    Option.getInstance();
	    Bprot.getInstance();
	    Baud.getInstance();
	    Sccr1.getInstance();
	    Sccr2.getInstance();
	    Scsr.getInstance();
	    Scdr.getInstance();
	    Coprst.getInstance();
	    //	    TOC1hi.getInstance();
	    for(int i = 0x1036; i < 0x1040; i++) {
		mem.setCell(i, new UnimplementedByteCell(i));
	    }
	    PortE.getInstance();
        Pctla.getInstance();
	} else {
	    System.out.println("SHOULD NOT HAPPEN");
	    System.exit(-1);
	}
    }

    /** Configure contiguous memory as RAM.
     * @param start Start of the RAM segment.
     * @param end  End of the RAM segment.
     */
    void configureRAM(int start, int end)
    {
	assert(end >= start);
	for(int i = start; i <= end; i++) {
	    mem.setCell(i, new RAMByteCell(i));
	}
    }

    /** Configure contiguous memory as ROM.
     * @param start Start of the ROM segment.
     * @param end  End of the ROM segment.
     */
    void configureROM(int start, int end)
    {
	for(int i = start; i <= end; i++) {
	    mem.setCell(i, new ROMByteCell(i));
	}
    }

    /** Configure the memory map from a configuration file.
     * NOT IMPLEMENTED.
     */
    void setConfig(String configFileName)
    {
	System.err.println("Configuring from file " + configFileName);
    }

//     public void showConfig()
//     {
// 	if (configFile == null) {
// 	    System.out.println("Standard configuration used:");
// 	}
//    	System.out.print(configToString());
//     }

    /**
     * Returns a String representation of the short version/copyright
     * notice.
     */
    public String versionToString()
    {
	return versionInfo;
    }
    
    /** Returns a String representation of the configuration.
     *
     */

    public String configToString()
    {
	Collection v;
	String s = "";
	v =  getUsedMemAreas(0);
	for(Iterator iter = v.iterator(); iter.hasNext();) {
	    s += iter.next().toString();
	}
	return s;
    }

    /**
     * Loads Motorola S19 object format into engine's memory.
     * @param fileName Path of alleged *.s19 file
     * @throws NoS19FileException
     */
    public void loadS19(String fileName) throws NoS19FileException
    {
	try {
	    Reader reader = new FileReader(fileName);
	    loadS19(reader);
	    reader.close();
	} catch (FileNotFoundException e) {
	    throw new NoS19FileException(fileName);
	} catch (IOException e) {
	    System.out.println("Got unexpected exception: " + e);
	}
	    
    }

    public void loadS19(Reader reader) throws IOException
    {
	int ch;
	int contents;
	char [] buf = new char[10];
	String s;
	Cell cell;

	while((ch = reader.read()) != -1) {
	    while (ch != 'S') {
		ch = reader.read();
		if (ch == -1) break;
	    }
	    ch = reader.read();
	    if (ch == '0') continue;
	    if (ch == '9') break;
	    reader.read(buf, 0, 2);
	    s = new String(buf, 0, 2);
	    int count = Integer.parseInt(s, 16);
	    reader.read(buf, 0, 4);
	    s = new String(buf, 0, 4);
	    int address = Integer.parseInt(s, 16);
	    count -= 3;
	    while(count-- > 0) {
		reader.read(buf, 0, 2);
		s = new String(buf, 0, 2);
		contents = Integer.parseInt(s, 16);
		cell = mem.getCell(address);
		cell.set(contents);
		address++;
	    }
	}
    }

    /** padding length for step String...kludge */
    static final int DLEN = 18; 

    /**
     * Single steps the CPU and returns a String summary of its effects.
     *
     * @return summary
     */

    public String stepUI()
    {
	InstructionEffects eff;
	String s;
	s = Utility.int16ToHex(cpu.PC) + " ";
	eff = cpu.step();
	s += Utility.padString(eff.disassembled, DLEN);
	s += Sim6811.CPUtoString();
	s += "\n";
	return s;
    }

    final int UNUSED = 0;
    final int RAM = 1;
    final static int ROM = 2;
    final static int SPECIAL = 3;
    final static int UNIMPLEMENTED = 4;
    /** Returns a collection of MemArea's.
     */

    private  Collection getUsedMemAreas(int start)
    {
	Vector v = new Vector();
	MemArea area = new MemArea();
	area.start = start;
	int type=0, end=start;
	String [] cellTypes = {"Unused",
			       "RAM",
			       "ROM",
			       "Special",
			       "Unimplemented"};
	//Get type of start cell
	Cell cell = mem.getCell(start);
	if(cell instanceof UnusedByteCell) {
	    type = UNUSED;
	    end = countUnused(start);
	} else if (cell instanceof RAMByteCell) {
	    type = RAM;
	    end = countRAM(start);
	} else if (cell instanceof ROMByteCell) {
	    type = ROM;
	    end = countROM(start);
	} else if (cell instanceof SpecialByteCell) {
	    type = SPECIAL;
	    end = countSpecial(start);
	} else if (cell instanceof UnimplementedByteCell) {
	    type = UNIMPLEMENTED;
	    end = countUnimplemented(start);
	} else {
	    System.out.println("Oops!...unknown cell at " +
			       Utility.int16ToHex(start));
	    System.exit(1);
	}
	area.end = end;
	area.type = type;
	area.typeName = cellTypes[type];
	v.add(area);
	if(end < 0xffff) {
	    v.addAll(getUsedMemAreas(end+1));
	}
	return v;
    }

    private int countRAM(int start)
    {
	int i;
	for(i = start;
	    i < 0x10000 && (mem.getCell(i) instanceof RAMByteCell);
	    i++)
	    ;
	return i-1;
    }

    private int countSpecial(int start)
    {
	int i;
	for(i = start;
	    i < 0x10000 && (mem.getCell(i) instanceof SpecialByteCell);
	    i++)
	    ;
	return i-1;
    }

    private int countROM(int start)
    {
	int i;
	for(i = start;
	    i < 0x10000 && (mem.getCell(i) instanceof ROMByteCell);
	    i++)
	    ;
	return i-1;
    }

    private int countUnused(int start)
    {
	int i;
	for(i = start;
	    i< 0x10000 && (mem.getCell(i) instanceof UnusedByteCell);
	    i++)
	    ;
	return i-1;
    }


    private int countUnimplemented(int start)
    {
	int i;
	for(i = start;
	    i< 0x10000 && (mem.getCell(i) instanceof UnimplementedByteCell);
	    i++)
	    ;
	return i-1;
    }

    /** Represent a contiguous segment of memory occupied by the
     * same type of Cell.
     */
    private class MemArea
    {
	int start;
	int end;
	String typeName;
	int type;

	public String toString()
	{
	    if (type == SPECIAL) {
		String str = "";
		for(int i = start; i <= end; i++) {
		    SpecialByteCell cell = (SpecialByteCell) mem.getCell(i);
		    str = str + Utility.int16ToHex(i) +
			"   " + cell.getName() + "\n";
		}
		return str;
	    } else {
		return Utility.int16ToHex(start) + "-"
		    + Utility.int16ToHex(end) + " "
		    + typeName + "\n";
	    }
	}
	    
    }

    //Following convenience methods to access LstLoader methods

    /** Parses a listing file, returning an array of <code>SourceLine</code>.
     *
     * @param fileName Name of file.
     * @throws NoLSTFileException
     */
    public SourceLine[] loadLST(String fileName) throws NoLSTFileException,
	NoDuplicateFileException
    {
	return lstLoader.loadLST(fileName);
    }

    /** Unloads the specified source code file. */
    public SourceLine [] unload(File file)
    {
	return lstLoader.unload(file);
    }

}

