package com.kclowes.sim11.ui;
import com.kclowes.sim11.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

/** A MemoryDump pane.
 *
 * <p>
 * The pane contains an editable  hex
 * dump of 8 bytes.
 * </p>
 *
 * <p>
 *  The pane has the format described in the following diagram where
 * the sample starting address of the dumps is 0x123A and the contents
 * of the 8 bytes (0x123A--0x1241) are assigned randomly:
 *<pre>
 *  +---------------------------------------------------+
 *  |Start Address :  123A            [start pane]      +
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *  |123.:  A  B  C  D  E  F  0  1    [header pane]     |
 *  +---------------------------------------------------+
 *  +---------------------------------------------------+
 *  |      01 23 45 67 89 AB CD EF    [content pane]    |
 *  +---------------------------------------------------+
 * </pre>
 * </p>
 *  @version 1.0 (October 13, 2002)
 *  @author Ken Clowes
 */

public class MemoryDumpSwing extends JPanel implements Refreshable
{
    private int startingAddress;
    private static Dimension CONTENT_PREF_SIZE = new Dimension(25, 20);
    private static Dimension CONTENT_MAX_SIZE = new Dimension(25, 20);
    private static Font HEADER_OFFSET_FONT =
	new Font("Courier", Font.BOLD, 14);
    private static Font CONTENT_FONT = new Font("Courier", Font.PLAIN, 14);
    final static char [] HexDigits = {'0', '1', '2', '3',
				      '4', '5', '6', '7',
				      '8', '9', 'A', 'B',
				      'C', 'D', 'E', 'F'};

    private JLabel[] headerLabels = new JLabel[8];
    private JTextField [] contentFields = new JTextField[8];
    private JLabel headerLabelStart;
    private MemorySpace mem = MemorySpace64K.getInstance();

    /** Create a memory dump pane at the given starting address.
     *
     * @param start The starting address of the dump.
     */

    MemoryDumpSwing(int start) {
	super();
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	startingAddress = start;

	add(createStartAddressPane());
	add(createColumnHeaders());
	add(createContentPane());
	    
	
    }

    private JComponent createStartAddressPane()
    {
	JPanel panel = new JPanel(new BorderLayout());
	
	panel.add(new JLabel("Start address: "), BorderLayout.WEST);
	JTextField starting =
	    new JTextField(Utility.int16ToHex(startingAddress));
	starting.setFont(HEADER_OFFSET_FONT);
	starting.
	    addActionListener(
			      new ActionListener() {
				  
				  public void actionPerformed(ActionEvent e)
				  {
				      JTextField f =
					  (JTextField) (e.getSource());
				      try {
					  int newStart = 
					      Integer.parseInt(
							       f.getText(),16);
					  if (newStart != startingAddress) {
					      startingAddress = newStart;
					      updateHeaders();
					      updateContents();
					  }
				      } catch (NumberFormatException fmt) {
					  String oldString =
					      Utility.int16ToHex(
							 startingAddress);
					  //badHex(oldString, f);
					  System.out.println("Bad hex fmt");
				      }
				  }
			      });
	
	panel.add(starting, BorderLayout.EAST);
	return panel;
    }

    private JComponent createColumnHeaders()
    {
	Box box = Box.createHorizontalBox();
	headerLabelStart = new JLabel("???.  ");
	headerLabelStart.setFont(HEADER_OFFSET_FONT);
	box.add(headerLabelStart);
	for (int i = 0; i < 8; i++) {
	    headerLabels[i] = new JLabel();
	    headerLabels[i].setFont(HEADER_OFFSET_FONT);
	    box.add(headerLabels[i]);
	}
	updateHeaders();
	return box;
    }


    private void updateHeaders()
    {
	String adr = Utility.int16ToHex(startingAddress);
	String adrFirstPart = adr.substring(0, 3) + "-";
	headerLabelStart.setText(adrFirstPart);
	
	for(int i = 0; i < 8; i++) {
	    char offsetChar = HexDigits[(((startingAddress%16) + i)%16)];
	    char [] chars = {' ', offsetChar, ' '};
	    String offsetString = new String(chars);
	    headerLabels[i].setText(offsetString);
	}
    }

    private JComponent createContentPane()
    {
	Box box;
	box = Box.createHorizontalBox();
	box.add(box.createHorizontalStrut(30));
	for(int i = 0; i < 8; i++) {
	    contentFields[i] = new JTextField("??");
	    contentFields[i].setFont(CONTENT_FONT);
	    contentFields[i].setPreferredSize(CONTENT_PREF_SIZE);
	    contentFields[i].setMaximumSize(CONTENT_MAX_SIZE);
	    contentFields[i].
		addActionListener(createContentListener(i));
	    //Oct 21, 2003 (kclowes)
	    contentFields[i].
		addMouseListener(createMouseListener(i));

	    box.add(contentFields[i]);
	}
	updateContents();
	return box;
    }
    
    private void updateContents()
    {
	String data;
	int contents;
	for(int i = 0; i < 8; i++) {
	    Cell cell = mem.getCell(startingAddress + i);
	    if (cell != null) {
		contents = cell.get();
		data = Utility.int8ToHex(contents);
	    } else {
		data = "??";
	    }
	    contentFields[i].setText(data);
	}
    }
    
    public void update()
    {
	updateContents();
    }



    private ActionListener createContentListener(int i)
    {
	return new ContentListener(i);
    }

    class ContentListener implements ActionListener
    {
	private int index;

	ContentListener(int i)
	{
	    index = i;
	}
	
	public void actionPerformed(ActionEvent e)
	{
	    int data;
	    int address = startingAddress + index;
	    Cell cell = mem.getCell(address);
	    JTextField f =
		(JTextField) (e.getSource());
	    try {
		data = Integer.parseInt(f.getText(),16);
		cell.set(data);
	    } catch (NumberFormatException fmt) {
		String oldString =
		    Utility.int8ToHex(
				      startingAddress);
		//badHex(oldString, f);
		System.
		    out.
		    println("Content Bad hex fmt");
	    }
	}
    }

    private MouseListener createMouseListener(int i)
    {
	return new MyMouseListener(i);
    }

    class MyMouseListener extends MouseAdapter
    {
	private int index;

	MyMouseListener(int i)
	{
	    index = i;
	}
	
	public void mouseExited(MouseEvent e)
	{
	    System.out.println("Mouse exit field: " + index);
	    System.out.println("focusOwner: "
			       + contentFields[index].isFocusOwner());
	    if (contentFields[index].isFocusOwner()) {
		System.out.println("getText(): "
			       + contentFields[index].getText());
		update();
	    }
	}
    }
}

