package com.kclowes.sim11.ui;
import com.kclowes.sim11.*;
import com.kclowes.sim11.devices.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
/** Creates a frame for eebot that provides status information about
 * the current state of the eebot.  
 * <p>The layout of the frame is:
 * <pre>
 *  +---------------------------------------------------+
 *  |              TITLE                                |
 *  +---------------------------------------------------+
 *  +--Motors-------------------------------------------+
 *  | Port: ON/OFF FWD/REV   Starboard: ON/OFF FWD/REV  |
 *  +---------------------------------------------------+
 *  +-Bumpers-------------------------------------------+
 *  |   Bow: status button   Stern: status button       |
 *  +---------------------------------------------------+
 *  +---Position----------------------------------------+
 *  |     x:     y:   Orientation:                      |
 *  +---------------------------------------------------+
 *</pre>
 *
 * <p> Details of each pane are described below.
 * </p>
 *
 * <center><b>Motors</b></center>
 * <p>
 *  The <em>Motors</em> pane provides status information about the
 *  Starboard (right) and Port (left) motors.  The information
 *  shows whether the motor is on or off and its direction.  In the
 *  current implementation, there are no facilities for
 *  <em>modifying</em> the state of a motor with the user interface.
 * </p>
 *
 * <center><b>Bumpers</b></center>
 * <p>
 *    The <em>Bumpers</em> pane provides status buttons (hit/not hit) for
 * each bumper.  Clicking a bumper status button changes the state of the
 * bumper and sets the analog voltage on the appropriate Port E pin.
 * Other clients may be listening for voltage changes on these pins; their
 * <code>perform()</code> will be invoked.
 * </p>
 *
 * <p> The <code>Robot</code> class also listens for changes to Port E
 * analog voltage changes and sets the bumper status buttons to their
 * proper states.
 * </p>
 *  @see com.kclowes.sim11.devices.PortE
 *  @version 1.1 (November 18, 2002)
 *  @author Ken Clowes
 */

public class Robot extends JFrame implements WindowListener
{
    static private final boolean DEBUG = false;
    CPU6811 cpu = CPU6811.getInstance();
    private PortA portA = PortA.getInstance();
    private PortE portE = PortE.getInstance();
    private JToggleButton bowBumperButton;
    private JToggleButton sternBumperButton;
    private MemorySpace mem = MemorySpace64K.getInstance();
    private Cell gpout = mem.getCell(0x1100);
    final String MOTOR_ON_STRING = "ON";
    final String MOTOR_OFF_STRING = "OFF";
    final String MOTOR_FWD_STRING = "FWD";
    final String MOTOR_REV_STRING = "REV";
    final int BOW_BUMPER_AD_CHANNEL = 2; //CHECK
    final int STERN_BUMPER_AD_CHANNEL = 3; //CHECK
    final int BUMPER_HIT_VOLTAGE = 0x10;   //Should be more symbolic
    final int BUMPER_NOT_HIT_VOLTAGE = 0xC0;   //Should be more symbolic
    final String BUMPER_NOT_HIT_STRING = "OK";
    final String BUMPER_HIT_STRING = "Hit";



    //Robot parameters
    final int RECALCULATE_POSITION = RobotParameters.RECALCULATE_POSITION;
    final int RECALCULATE_ORIENTATION =
	RobotParameters.RECALCULATE_ORIENTATION;
    final int CYCLES_PER_TICK = RobotParameters.CYCLES_PER_TICK;
    final double BOT_SPEED = RobotParameters.SPEED;
    final double BOT_DIAMETER = RobotParameters.DIAMETER;
    final double BOT_RADIUS = BOT_DIAMETER/2.0;
    final double TICKS_PER_METER = RobotParameters.TICKS_PER_METER;
    
    //Robot state variables
    private boolean starMotorON = false;
    private boolean portMotorON = false;
    private boolean prevStarMotorON = false;
    private boolean prevPortMotorON = false;
    private boolean starFWD = true;
    private boolean portFWD = true;
    private int starTicks = 0;
    private int portTicks = 0;
    private double botOrientation = 0.0; //Orientation (radians) of bot
    private double botX = 0.0;
    private double botY = 0.0;
    
    private JLabel stateStarLabel = new JLabel(MOTOR_OFF_STRING);
    private JLabel dirStarLabel = new JLabel(MOTOR_REV_STRING);
    private JLabel statePortLabel = new JLabel(MOTOR_OFF_STRING);
    private JLabel dirPortLabel = new JLabel(MOTOR_REV_STRING);

    private JLabel xPositionLabel = new JLabel();
    private JLabel yPositionLabel = new JLabel();
    private DecimalFormat positionFormat = new DecimalFormat("##0.00");
    private JLabel orientationLabel = new JLabel();
    private DecimalFormat orientationFormat = new DecimalFormat("##0.0");

    Robot()
    {
// 	System.out.println("Bot speed (cm/sec): " + BOT_SPEED*100);
// 	System.out.println("Bot diameter (cm): " + BOT_DIAMETER*100);
// 	System.out.println("Ticks per millimeter: " +
// 			   (int)(TICKS_PER_METER/1000));
// 	System.out.println("Recalc position after ticks : " +
// 			   RECALCULATE_POSITION);
// 	System.out.println("Recalc orientation if diff >= " +
// 			   RECALCULATE_ORIENTATION);
	setTitle("EEBOT");
	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        getContentPane().add(createRobotPanel());
	addWindowListener(this);
	pack();
	setLocation(950, 500);
	setVisible(true);
    }

    JComponent createRobotPanel()
    {
	JPanel panel = new JPanel(new BorderLayout());
	JPanel motors = createMotorsPane();
	panel.add(motors, BorderLayout.NORTH);
	JPanel bumpers = createBumpersPane();
	panel.add(bumpers, BorderLayout.CENTER);
	JPanel position = createPositionPane();
	panel.add(position, BorderLayout.SOUTH);
	
	return panel;
    }

    JPanel createMotorsPane()
    {
        String title;
        JPanel panel = new JPanel(new BorderLayout());
        title = "Motors";
        panel.setBorder(
        BorderFactory.createTitledBorder(title));
        JComponent starboard = createStarboardPane();
        panel.add(starboard, BorderLayout.NORTH);
        JComponent port = createPortPane();
        panel.add(port, BorderLayout.SOUTH);
        portA.addCellChangeListener(new CellChangeListener() {
            public void perform(Cell c)
            {
                starMotorON = (portA.get()&0x20) == 0x20;
                //Has the starboard motor just turned on?
                if (starMotorON && !prevStarMotorON) {
                    cpu.addTimedEvent(
                    new TimedEvent(CYCLES_PER_TICK,
                    new StarMotorOnAction()));
                }
                prevStarMotorON = starMotorON;
                
                portMotorON = (portA.get()&0x10) == 0x10;
                if (portMotorON && !prevPortMotorON) {
                    cpu.addTimedEvent(
                    new TimedEvent(CYCLES_PER_TICK,
                    new PortMotorOnAction()));
                }
                prevPortMotorON = portMotorON;
                
                stateStarLabel.setText(starMotorON ?
                MOTOR_ON_STRING : MOTOR_OFF_STRING);
                statePortLabel.setText(portMotorON ?
                MOTOR_ON_STRING : MOTOR_OFF_STRING);
            }
        });
	
    gpout.addCellChangeListener(new CellChangeListener() {
        public void perform(Cell c)
        {
            starFWD = (gpout.get()&0x02) != 0x02;
            portFWD = (gpout.get()&0x01) != 0x01;
            dirStarLabel.setText(starFWD ?
            MOTOR_FWD_STRING : MOTOR_REV_STRING);
            dirPortLabel.setText(portFWD ?
            MOTOR_FWD_STRING : MOTOR_REV_STRING);
        }
    });
    return panel;
    }
    
    private JComponent createStarboardPane()
    {
        JPanel panel = new JPanel(new BorderLayout());
        JLabel label = new JLabel("Starboard: ");
        panel.add(label, BorderLayout.WEST);
        panel.add(stateStarLabel, BorderLayout.CENTER);
        panel.add(dirStarLabel, BorderLayout.EAST);
        
        return panel;
    }
    
    
    private JComponent createPortPane()
    {
        JPanel panel = new JPanel(new BorderLayout());
        JLabel label = new JLabel("Port: ");
        panel.add(label, BorderLayout.WEST);
        panel.add(statePortLabel, BorderLayout.CENTER);
        panel.add(dirPortLabel, BorderLayout.EAST);
        
        return panel;
    }
    
    JPanel createBumpersPane()
    {
        String title;
        JPanel panel = new JPanel(new BorderLayout());
        title = "Bumpers";
        panel.setBorder(
        BorderFactory.createTitledBorder(title));
        
        JComponent bowBumperPane = createBowBumper();
        panel.add(bowBumperPane, BorderLayout.WEST);
        JComponent sternBumperPane = createSternBumper();
        panel.add(sternBumperPane, BorderLayout.EAST);
        
        //Listen for PortE Analog Voltage Events
        //One voltage corresponds to the Bow Bumper sensor;
        //another to the Stern Bumper sensor
        
        portE.addVoltageChangeListener(new VoltageChangeListener() {
            public void perform(PortE pe)
            {
                int [] voltages;
                voltages = portE.getVoltages();
                assert(voltages.length == 8);
                
                if (voltages[BOW_BUMPER_AD_CHANNEL]
                < BUMPER_NOT_HIT_VOLTAGE) {
                    bowBumperButton.setText(BUMPER_HIT_STRING);
                    bowBumperButton.setSelected(true);
                } else {
                    bowBumperButton.setText(BUMPER_NOT_HIT_STRING);
                    bowBumperButton.setSelected(false);
                }
                
                if (voltages[STERN_BUMPER_AD_CHANNEL]
                < BUMPER_NOT_HIT_VOLTAGE) {
                    sternBumperButton.setText(BUMPER_HIT_STRING);
                    sternBumperButton.setSelected(true);
                } else {
                    sternBumperButton.setText(BUMPER_NOT_HIT_STRING);
                    sternBumperButton.setSelected(false);
                }
            }
        });
        
	
	//Set voltages on PortE to "not hit"
	int v[] = portE.getVoltages();
	v[BOW_BUMPER_AD_CHANNEL] = BUMPER_NOT_HIT_VOLTAGE;
	v[STERN_BUMPER_AD_CHANNEL] = BUMPER_NOT_HIT_VOLTAGE;
	portE.setVoltages(v);
	return panel;
    }
    
    JPanel createBowBumper()
    {
	JPanel panel = new JPanel(new BorderLayout());
	JLabel label = new JLabel("Bow: ");
	panel.add(label, BorderLayout.WEST);
	bowBumperButton =
	    new JToggleButton(new AbstractAction() {
		    public void actionPerformed(ActionEvent e)
		    {
			int [] volts;
			String text;
			int v;
			
			if (bowBumperButton.isSelected()) {
			    text = BUMPER_HIT_STRING;
			    v = BUMPER_HIT_VOLTAGE;
			} else {
			    text = BUMPER_NOT_HIT_STRING;
			    v = BUMPER_NOT_HIT_VOLTAGE;
			}
			bowBumperButton.setText(text);
			volts = portE.getVoltages();
			volts[BOW_BUMPER_AD_CHANNEL] = v;
			portE.setVoltages(volts);
		    }
		});
	bowBumperButton.setText(BUMPER_NOT_HIT_STRING);
	panel.add(bowBumperButton, BorderLayout.EAST);
	return panel;
    }
    
    JPanel createSternBumper()
    {
	JPanel panel = new JPanel(new BorderLayout());
	JLabel label = new JLabel("Stern: ");
	panel.add(label, BorderLayout.WEST);
	sternBumperButton =
	    new JToggleButton(new AbstractAction() {
		    public void actionPerformed(ActionEvent e)
		    {
			int [] volts;
			String text;
			int v;
			
			if (sternBumperButton.isSelected()) {
			    text = BUMPER_HIT_STRING;
			    v = BUMPER_HIT_VOLTAGE;
			} else {
			    text = BUMPER_NOT_HIT_STRING;
			    v = BUMPER_NOT_HIT_VOLTAGE;
			}
			sternBumperButton.setText(text);
			volts = portE.getVoltages();
			volts[STERN_BUMPER_AD_CHANNEL] = v;
			portE.setVoltages(volts);
		    }
		});
	sternBumperButton.setText(BUMPER_NOT_HIT_STRING);
	panel.add(sternBumperButton, BorderLayout.EAST);
	return panel;
    }

    JPanel createPositionPane()
    {
	String title;
	JPanel panel = new JPanel(new BorderLayout());
	title = "Position";
	panel.setBorder(
			BorderFactory.createTitledBorder(title));

	JComponent xPositionPane = createXPositionPane();
	panel.add(xPositionPane, BorderLayout.WEST);
	JComponent yPositionPane = createYPositionPane();
	panel.add(yPositionPane, BorderLayout.CENTER);
	JComponent orientationPane = createOrientationPane();
	panel.add(orientationPane, BorderLayout.EAST);
	return panel;
    }

    private JComponent createXPositionPane()
    {
	JPanel panel = new JPanel(new BorderLayout());
	JLabel label = new JLabel("X: ");
	panel.add(label, BorderLayout.WEST);
	xPositionLabel.setText(positionFormat.format(botX));
	panel.add(xPositionLabel, BorderLayout.EAST);
	
	return panel;
    }
    
    private JComponent createYPositionPane()
    {
	JPanel panel = new JPanel(new BorderLayout());
	JLabel label = new JLabel(" Y: ");
	panel.add(label, BorderLayout.WEST);
	yPositionLabel.setText(positionFormat.format(botY));
	panel.add(yPositionLabel, BorderLayout.EAST);
	
	return panel;
    }
    
	private JComponent createOrientationPane()
    {
	JPanel panel = new JPanel(new BorderLayout());
	JLabel label = new JLabel("Angle: ");
	panel.add(label, BorderLayout.WEST);
	setOrientationUI();
	panel.add(orientationLabel, BorderLayout.EAST);
	
	return panel;
    }
    

    /** Recalculates eeBot position.  This function should only be
    invoked  if either motor has "ticked" more/equal than
    <code>RECALCULATE_POSITION</code> times or if the difference
    in ticks is more/equal than <code>RECALCULATE_ORIENTATION</code>.
    (This is enforced with
    an assertion.)
    */
    private void recalculatePosition()
    {
	assert ((Math.abs(starTicks) >= RECALCULATE_POSITION) ||
		(Math.abs(portTicks) >= RECALCULATE_POSITION) ||
		(Math.abs(starTicks - portTicks) >= RECALCULATE_ORIENTATION)) :
	    "starTicks: " + starTicks + " portTicks: " + portTicks +
	    "diff: " + (starTicks-portTicks);

	//Difference proportional to rotation in clockwise direction
	int diffTicks = (starTicks - portTicks)/2;
	//
	int distanceMovedTicks = (starTicks + portTicks)/2;
	//Split straight-line distance into x and y components
	double deltaXTicks = distanceMovedTicks*Math.cos(botOrientation);
	double deltaYTicks = distanceMovedTicks*Math.sin(botOrientation);
	double deltaXMeters = deltaXTicks/TICKS_PER_METER;
	double deltaYMeters = deltaYTicks/TICKS_PER_METER;
	botX += deltaXMeters;
	botY += deltaYMeters;
	//	if (Math.abs(starTicks-portTicks) >= RECALCULATE_ORIENTATION) {
	if (diffTicks != 0) {
	    double diffMeters = diffTicks/TICKS_PER_METER;
	    double deltaOrientation = Math.atan2(diffMeters, BOT_RADIUS);
// 	    System.out.println("Recalculating eeBot orientation, diff: " +
// 			       diffMeters + " m " +
// 			       (int) (diffMeters*10000) + " mm*10 " +
// 			       "\n  BOT_RADIUS: " + (int)(BOT_RADIUS*1000) +
// 			       " mm ");
	    botOrientation += deltaOrientation;
	    //Normalize botOrientation 0--360 degrees
	    while (botOrientation > Math.PI*2) {
		botOrientation -= (Math.PI*2);
	    }
	    while (botOrientation < -Math.PI*2) {
		botOrientation += (Math.PI*2);
	    }
	    if (botOrientation < 0.0) {
		botOrientation += (Math.PI*2);
	    }
	}
	xPositionLabel.setText(positionFormat.format(botX));
	yPositionLabel.setText(positionFormat.format(botY));
	double orientationDegrees = Math.toDegrees(botOrientation);
// 	System.out.println("x,y: " + positionFormat.format(botX) + ", " +
// 			   positionFormat.format(botY) +
// 			   " Angle: " +
// 			   orientationFormat.format(orientationDegrees) +
// 			   " degrees");
	setOrientationUI();
	starTicks = 0;
	portTicks = 0;
    }

    private void setOrientationUI()
    {
	double orientationDegrees = Math.toDegrees(botOrientation);
	String formattedOrientationDegrees =
	    orientationFormat.format(orientationDegrees);
								      
	orientationLabel.setText(formattedOrientationDegrees);
    }

    public void windowClosing(WindowEvent e)
    {
	System.out.println("Closing Robot frame.");
    }
    
    public void windowOpened(WindowEvent e)
    {
    }
    public void windowClosed(WindowEvent e)
    {
    }
    public void windowIconified(WindowEvent e)
    {
    }
    public void windowDeiconified(WindowEvent e)
    {
    }
    public void windowActivated(WindowEvent e)
    {
    }
    public void windowDeactivated(WindowEvent e)
    {
    }
    
    class  StarMotorOnAction implements TimedAction{
	public void doAction() {
	    if (DEBUG) {
		System.out.println("Star motor ticks: " + starTicks);
	    }
	    if (starFWD) {
		starTicks++;
	    } else {
		starTicks--;
	    }
	    if ((Math.abs(starTicks) >= RECALCULATE_POSITION) ||
		(Math.abs(starTicks - portTicks) >= RECALCULATE_ORIENTATION)){
		recalculatePosition();
	    }
	    if (starMotorON) {
		cpu.addTimedEvent(new TimedEvent(CYCLES_PER_TICK,
						 new StarMotorOnAction()));
	    }
	}
	
    }

    class  PortMotorOnAction implements TimedAction{
	public void doAction() {
	    if (DEBUG) {
		System.out.println("Port motor ticks: " + portTicks);
	    }
	    if (portFWD) {
		portTicks++;
	    } else {
		portTicks--;
	    }
	    if ((Math.abs(portTicks) >= RECALCULATE_POSITION) ||
		(Math.abs(starTicks - portTicks) >= RECALCULATE_ORIENTATION)){
		recalculatePosition();
	    }
	    if (portMotorON) {
		cpu.addTimedEvent(new TimedEvent(CYCLES_PER_TICK,
						 new PortMotorOnAction()));
	    }
	}
	
    }

    static public void main(String args[])
    {
	Robot mainFrame = new Robot();
    }
}
